package org.ow2.choreos.sparqlfunctions.constants;

import java.util.ArrayList;
import org.apache.commons.math3.optimization.fitting.*;
import org.apache.commons.math3.analysis.polynomials.PolynomialFunction;
import org.apache.commons.math3.analysis.polynomials.PolynomialFunction.Parametric;
import java.util.Date;
import java.util.HashMap;
import java.util.Map.Entry;

import org.ow2.chereos.iots.common.Function;
import org.ow2.chereos.iots.common.Location;
import org.ow2.choreos.iots.datatypes.DefaultDoubleData;
import org.ow2.choreos.iots.datatypes.SensorData;
import org.ow2.choreos.query.impl.MathematicaClient;
import org.ow2.choreos.query.impl.SubQuery;
import org.ow2.choreos.sparqlfunctions.impl.SparqlImpl;
import org.ow2.choreos.sparqlfunctions.interfaces.Sparql;


import com.hp.hpl.jena.graph.query.Expression.Constant;

public class DataFusionGeneratorImpl {

	
	/**
	 * This method is called after data is acquired from devices to perform
	 * fusion/aggregation and returns the result. This method assumes there are
	 * no substitution queries, and the request is only to take a set of data
	 * and apply the fusion function to them
	 * 
	 * @param dataPerServiceMap
	 *            : a HashMap containing the measurement for each service
	 *            address
	 * @param fusionFunction
	 *            : mathematical functions that can be use the substitute
	 *            queries to compute the needed result
	 * @return the result
	 */
	public SensorData generateFinalDataValues(
			HashMap<String, SensorData> dataPerServiceMap, String fusionFunction)
			throws NumberFormatException {
		SensorData finalDataVal = null;
		String unit = null;
		Unit requestedUnit = null;
		if (fusionFunction.equals("average")) {
			// TODO maybe send parameters here
			ArrayList<SensorData> dataValues = new ArrayList<SensorData>();
			if (dataPerServiceMap != null) {
				for (Entry<String, SensorData> deviceEntry : dataPerServiceMap
						.entrySet()) {

					dataValues.add(deviceEntry.getValue());

				}
				finalDataVal = new DefaultDoubleData(Function.getAverage(
						dataValues, dataPerServiceMap.size()),
						new Date().getTime());

			} else
				finalDataVal = new DefaultDoubleData(0, new Date().getTime());

		} else if (fusionFunction.equals("first")) {
			ArrayList<SensorData> dataValues = new ArrayList<SensorData>();
			for (Entry<String, SensorData> deviceEntry : dataPerServiceMap
					.entrySet()) {
				// TODO add conversion code here too
				dataValues.add(deviceEntry.getValue());
				SensorData sd = dataValues.get(0);
				finalDataVal = sd;
			}
		} else {
			System.out
					.println("DataFusionGenerator.generateFinalDataValues: function type is wrong");
			return null;
		}

		return finalDataVal;
	}

	
	// TODO maybe i should remove the subQueries object, i dont use it
	/**
	 * This method is called after data is acquired from devices and expansion formulas are computed.
	 * the result is returned as a double for each formula, for which the method computes the requested aggregation
	 * The method assumes all results have already been converted to the same unit
	 * 
	 * @param dataPerFormulaMap
	 *            : a HashMap containing the results of each expansion formula
	 * @param fusionFunction
	 *            : mathematical functions that can be use the substitute
	 *            queries to compute the needed result
	 * @return the result
	 */
	public double generateFinalDataValues(
			HashMap<Formula, Double> dataPerFormulaMap,
			HashMap<SubQuery, Formula> mySubQueries, String fusionFunction) {
		double finalDataVal = 0;

		Unit requestedUnit = null;
		if (fusionFunction.equals("average")) {
			// for (Entry<SubQuery, Formula> entry : mySubQueries.entrySet()) {
			ArrayList<Double> dataValues = new ArrayList<Double>();
			if (dataPerFormulaMap != null) {
				for (Entry<Formula, Double> deviceEntry : dataPerFormulaMap
						.entrySet()) {
					Formula thisFormula = deviceEntry.getKey();

					dataValues.add(deviceEntry.getValue());

				}

				finalDataVal = Function.getAverageDouble(dataValues,
						dataPerFormulaMap.size());

			} else
				finalDataVal = 0;

			// }
		} else if (fusionFunction.equals("first")) {
			ArrayList<Double> dataValues = new ArrayList<Double>();
			for (Entry<Formula, Double> deviceEntry : dataPerFormulaMap
					.entrySet()) {

				dataValues.add(deviceEntry.getValue());
			}
			finalDataVal = dataValues.get(0);

		} else {
			System.out
					.println("DataFusionGenerator.generateFinalDataValues: function type is wrong");
			return 0;
		}

		return finalDataVal;
	}

	public double generateFinalDataValues(ArrayList<Measurement> measurements,
			Concept mainConcept, Location mainLoc, String fusionFunction) {
		
		double finalDataVal = 0;
		if (fusionFunction.equals("average")){
	         int i;
	          double valueHolder = 0;
	          for(i=0;i<measurements.size();i++) {
	              valueHolder=valueHolder+measurements.get(i).getMeasurementValue();
	          }
	          return valueHolder/i;
		}
		if (fusionFunction.equals("fitToCurve")) {
			String commands = "" + measurements.size() + " ";
			String commandsDist= "";
			String commandsVal= "";
			SparqlImpl sp = new SparqlImpl();

			for (int i = 0; i < measurements.size(); i++) {
				Location thisLoc = measurements.get(i).getMeasurementLocation();
			
				double distance = Math.sqrt(Math.pow(
						(mainLoc.getXCoordinate() - thisLoc.getXCoordinate()),
						2)
						+ Math.pow((mainLoc.getYCoordinate() - thisLoc
								.getYCoordinate()), 2));

				commandsDist = commandsDist + ""+distance + " ";
				commandsVal = commandsVal + measurements.get(i).getMeasurementValue() + " ";
			}
		
			commands = commands + "" + commandsDist + "" + commandsVal;
			
			String formulaExpression = sp
					.getDecayFormulaExpression(mainConcept);
			formulaExpression = formulaExpression.replaceAll(
					"decayFactor",
					""
							+ DecayConstants.getDecayValue(mainConcept
									.getConceptName()));
			formulaExpression = formulaExpression.replace("(",	"[");
			formulaExpression = formulaExpression.replace(")",	"]");
			
			commands = commands + formulaExpression;
			finalDataVal = MathematicaClient.getValue(commands);

		}
		return finalDataVal;
	}
}