package queryTest;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.HashMap;

import org.junit.Before;
import org.junit.Test;
import org.ow2.chereos.iots.common.Function;
import org.ow2.chereos.iots.common.Location;
import org.ow2.choreos.iots.datatypes.DefaultDoubleData;
import org.ow2.choreos.iots.datatypes.SensorData;
import org.ow2.choreos.query.constants.Selector;
import org.ow2.choreos.query.constants.Setter;
import org.ow2.choreos.query.constants.Where;
import org.ow2.choreos.query.impl.Query;
import org.ow2.choreos.sparqlfunctions.constants.Concept;
import org.ow2.choreos.sparqlfunctions.constants.DataFusionGeneratorImpl;
import org.ow2.choreos.sparqlfunctions.constants.Measurement;

import de.congrace.exp4j.UnknownFunctionException;
import de.congrace.exp4j.UnparsableExpressionException;

public class DataFusionGeneratorImplTest {
	DataFusionGeneratorImpl df;
	HashMap<String, SensorData> dataPerServiceMap;
	String addressService1, addressService2;
	ArrayList<Concept> conceptsToMeasure;
	HashMap<Query, Function> mySubQueries;
	Query myQuery;

	@Before
	public void setUp() throws Exception {
		df = new DataFusionGeneratorImpl();
		dataPerServiceMap = new HashMap<String, SensorData>();
		SensorData s = new DefaultDoubleData(13, 0);
		SensorData sd = new DefaultDoubleData(14, 0);
		addressService1 = "www.test.com/address/service1";
		addressService2 = "www.test.com/address/service2";
		dataPerServiceMap.put(addressService1, s);
		dataPerServiceMap.put(addressService2, sd);

		conceptsToMeasure = new ArrayList<Concept>();
		conceptsToMeasure.add(new Concept("noiselevel"));
		Location locationOfInterest = new Location("Paris");
		myQuery = new Query(new Selector(conceptsToMeasure), new Setter(
				new Where("noiselevel.unit.equals=db")), locationOfInterest);

	}

	@Test
	/**
	 * This test assumes no composition is taking place for the average function
	 */
	public void testGenerateFinalDataValues() throws NumberFormatException,
			UnknownFunctionException, UnparsableExpressionException {
		System.out.println(df.generateFinalDataValues(dataPerServiceMap,
				"average"));
		assertTrue(
				"the average value is not correct",
				Double.parseDouble(df.generateFinalDataValues(
						dataPerServiceMap, "average").toString()) == 13.5);
	}

	@Test
	/**
	 * This test assumes no composition is taking place for the getting the first item function
	 */
	public void testGenerateFinalDataValues1() throws NumberFormatException,
			UnknownFunctionException, UnparsableExpressionException {

		assertTrue(
				"the first item is not correct",
				Double.parseDouble(df.generateFinalDataValues(
						dataPerServiceMap, "first").toString()) == 14);
	}

	@Test
	public void testGenerateFinalDataValue() {
		String fusionFunction = "fitToCurve";
		Query mainQuery = new Query(new Selector(new Concept("noiselevel")),
				new Setter(new Where("noiselevel.unit.equals=db")),
				new Location("Paris", 0, 0));
		ArrayList<Measurement> myMeasurements = new ArrayList<Measurement>();
		myMeasurements.add(new Measurement(34.7249, new Location("", 1, 1)));
		myMeasurements.add(new Measurement(30.1455, new Location("", 2, 2)));
		myMeasurements.add(new Measurement(26.17, new Location("", 3, 3)));
		myMeasurements.add(new Measurement(22.7188, new Location("", 4, 4)));
		myMeasurements.add(new Measurement(19.7227, new Location("", 5, 5)));
		myMeasurements.add(new Measurement(17.1218, new Location("", 6, 6)));
		myMeasurements.add(new Measurement(15.1704, new Location("", 7, 7)));

		assertEquals(" the returned value should be 40.04",
				df.generateFinalDataValues(myMeasurements, new Concept(
						"noiselevel"), new Location("Paris", 0, 0),
						fusionFunction), 40.04316336055223, 0);
	}
}
