/*******************************************************************************
 * Copyright (c) 2011 EBM Websourcing.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v2.1
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * 
 * Contributors:
 *     EBM Websourcing - initial API and implementation
 ******************************************************************************/
package com.ebmwebsourcing.easyesb.component.cd.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;

import org.choreos.genericcdinterface.AdditionalMessageType;
import org.jdom.input.DOMBuilder;
import org.petalslink.abslayer.Factory;
import org.petalslink.abslayer.service.api.Binding;
import org.petalslink.abslayer.service.api.Description;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easybox.api.XmlContext;
import com.ebmwebsourcing.easybox.api.XmlContextFactory;
import com.ebmwebsourcing.easybox.api.XmlObjectReadException;
import com.ebmwebsourcing.easybox.api.XmlObjectReader;
import com.ebmwebsourcing.easycommons.research.util.easybox.SOAUtil;
import com.ebmwebsourcing.easycommons.research.util.jaxb.SOAJAXBContext;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponentBehaviour;
import com.ebmwebsourcing.easyesb.constant.EasyESBFramework;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.component.Component;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ClientEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.behaviour.EndpointBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.node.NodeBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;
import com.ebmwebsourcing.easyesb.soa.impl.component.AbstractComponentBehaviourImpl;
import com.ebmwebsourcing.easyesb.soa.impl.endpoint.ClientEndpointImpl;
import com.ebmwebsourcing.easyesb.soa10.api.element.EndpointInitialContext;
import com.ebmwebsourcing.easyesb.soa10.api.type.ClientEndpointType;
import com.ebmwebsourcing.easyesb.soa10.api.type.ComponentType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import com.ebmwebsourcing.easywsdl11.api.element.Definitions;

import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeploy;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeployResponse;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbUndeploy;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbUndeployResponse;
import esstar.petalslink.com.data.management.admin._1.NodeInformation;
import esstar.petalslink.com.data.management.admin._1.Ping;
import esstar.petalslink.com.data.management.admin._1.PingResponse;
import esstar.petalslink.com.service.management._1_0.ManagementException;

/**
 * Class that provides the administration of the CD component, that is to say,
 * the deployment of CDs
 **/
public class CDComponentBehaviourImpl extends AbstractComponentBehaviourImpl implements CDComponentBehaviour {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(CDComponentBehaviourImpl.class.getName());

	private final File tmpFolder = new File(System.getProperty("java.io.tmpdir"));

	// private static CoordinationDelegateUtility primitives = null;

	private static XmlContextFactory xmlContextFactory;
	private static XmlContext xmlContext;
	private static XmlObjectReader reader;

	private static Binding b;

	static {

		try {
			// URL url =
			// Thread.currentThread().getContextClassLoader().getResource("wsdl/genericCDInterface.wsdl");
			URL url = Thread.currentThread().getContextClassLoader().getResource(CDComponentBehaviour.DESCRIPTION_URL);
			Description desc = (Description) Factory.getInstance().wrap(SOAUtil.getInstance().getReader(EasyESBFramework.getInstance()).get().readDocument(url, Definitions.class));
			b = desc.getBindings().iterator().next();

		} catch (XmlObjectReadException e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
	}

	public CDComponentBehaviourImpl(Component<? extends ComponentType> ep) {
		super(ep);
		this.setBinding(b);
		xmlContextFactory = new XmlContextFactory();
		xmlContext = xmlContextFactory.newContext();
		reader = xmlContext.createReader();

		// primitives = new CoordinationDelegateUtility(this);

	}

	/**
	 * Main EASYESB component operation called when a request is sent to the CD
	 * administration service. In particular it receives deployment requests.
	 **/
	public void execute(Exchange exchange) throws TransportException {
		try {
			if (exchange.getMessageIn().getBody().getPayload() == null) {
				throw new TransportException("the message in cannot be null");
			}

			// convert dom to jdom
			DOMBuilder builder = new DOMBuilder();
			org.jdom.Document doc = builder.build(exchange.getMessageIn().getBody().getPayload());

			/*
			 * Deploy a CD
			 */
			if (doc.getRootElement() != null && doc.getRootElement().getName().equals("deploy")) {
				log.finest("Store CD");

				EJaxbDeploy deploy = SOAJAXBContext.getInstance().marshallAnyType(exchange.getMessageIn().getBody().getPayload(), EJaxbDeploy.class);

				/* call to the internal deploy operation */
				EJaxbDeployResponse res = this.deploy(deploy);

				Document docResp = SOAJAXBContext.getInstance().unmarshallAnyElement(res);
				MessageUtil.getInstance().createOutMessageStructure(exchange);
				exchange.getMessageOut().getBody().setPayload(docResp);

			}
		} catch (Exception e) {
			throw new TransportException(e);
		}

	}

	@Override
	public PingResponse ping(Ping arg0) {
		PingResponse pong = new PingResponse();
		NodeInformation value = new NodeInformation();
		value.setName("CD Component on node " + this.getEndpoint().getNode().getName());
		value.setVersion("1.0-SNAPSHOT");
		pong.setNodeInformation(value);
		return pong;
	}

	public void createClientEndpoint(Holder<QName> clientEndpointName, String classClientEndpointName, String classClientEndpointBehaviourName) {
		Class<? extends ClientEndpointImpl> clazz;
		try {
			if (classClientEndpointName != null) {
				clazz = (Class<? extends ClientEndpointImpl<ClientEndpointType>>) Class.forName(classClientEndpointName);
			} else {
				clazz = (Class<ClientEndpointImpl>) ClientEndpointImpl.class;
			}

			// add behaviour
			Class<? extends EndpointBehaviour> clazzBehaviour = null;
			if (classClientEndpointBehaviourName != null && classClientEndpointBehaviourName.trim().length() > 0) {
				clazzBehaviour = (Class<? extends EndpointBehaviour>) Class.forName(classClientEndpointBehaviourName);
			}

			EndpointInitialContext context = SOAUtil.getInstance().getXmlContext(EasyESBFramework.getInstance()).getXmlObjectFactory().create(EndpointInitialContext.class);
			context.setNumberOfThreads(5);
			ClientEndpoint<ClientEndpointType> client = (ClientEndpoint<ClientEndpointType>) ((NodeBehaviour) this.getEndpoint().getNode().findBehaviour(NodeBehaviour.class)).createClientEndpoint(clientEndpointName.value, clazz, clazzBehaviour, context);
			client.addBehaviourClass(clazzBehaviour);

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void fillFile(File file, DataHandler dh) throws IOException {

		char[] buffer = new char[1024];
		Writer writer = new FileWriter(file);
		BufferedInputStream bis = new BufferedInputStream(dh.getInputStream());
		Reader reader = new BufferedReader(new InputStreamReader(bis, "UTF-8"));

		int n;
		while ((n = reader.read(buffer)) != -1) {

			writer.write(buffer, 0, n);

		}
		reader.close();
		bis.close();
		writer.flush();
		writer.close();
	}

	@Override
	public List<String> getSupportedResourcesExtensions() {
		List<String> res = new ArrayList<String>();
		res.add("lts");
		res.add("cd");
		res.add("conf");
		return res;
	}

	@Override
	public void start() throws ESBException {
		// TODO Auto-generated method stub

	}

	@Override
	public void stop() throws ESBException {
		// TODO Auto-generated method stub

	}

	@Override
	public CoordinationDelegateAlgorithm getCoordinationDelegateAlgorithm(CoordinationDelegateID coordinationDelegateID) {
		return null;
	}

	@Override
	public EJaxbUndeployResponse undeploy(EJaxbUndeploy arg0) throws ManagementException {
		return ((CDComponentDeployBehaviourImpl) this.endpoint.findBehaviour(CDComponentDeployBehaviourImpl.class)).undeploy(arg0);
	}

	@Override
	public CoordinationDelegateUtility getPrimitives() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EJaxbDeployResponse deploy(EJaxbDeploy arg0) throws ManagementException {
		try {
			if (this.getEndpoint().findBehaviour(CDComponentDeployBehaviourImpl.class) == null) {
				((Component) this.getEndpoint().getComponent().getFcInterface("service")).addBehaviourClass(CDComponentDeployBehaviourImpl.class);
			}

		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return ((CDComponentDeployBehaviourImpl) this.endpoint.findBehaviour(CDComponentDeployBehaviourImpl.class)).deploy(arg0);
	}

	@Override
	public void updatecurrentstate(AdditionalMessageType additionalMessage) {
		//Do nothing
		
	}


}
