/*******************************************************************************
 * Copyright (c) 2011 EBM Websourcing.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v2.1
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * 
 * Contributors:
 *     EBM Websourcing - initial API and implementation
 ******************************************************************************/
package com.ebmwebsourcing.easyesb.component.cd.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateData;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.impl.DummyCoordinationDelegateAlgorithmImpl;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Coord;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelParser;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.io.Writer;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URL;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.activation.DataHandler;
import javax.xml.namespace.QName;
import javax.xml.ws.Holder;

import org.jdom.input.DOMBuilder;
import org.oasis_open.docs.wsn.bw_2.InvalidFilterFault;
import org.oasis_open.docs.wsn.bw_2.InvalidMessageContentExpressionFault;
import org.oasis_open.docs.wsn.bw_2.InvalidProducerPropertiesExpressionFault;
import org.oasis_open.docs.wsn.bw_2.InvalidTopicExpressionFault;
import org.oasis_open.docs.wsn.bw_2.NotifyMessageNotSupportedFault;
import org.oasis_open.docs.wsn.bw_2.SubscribeCreationFailedFault;
import org.oasis_open.docs.wsn.bw_2.TopicExpressionDialectUnknownFault;
import org.oasis_open.docs.wsn.bw_2.TopicNotSupportedFault;
import org.oasis_open.docs.wsn.bw_2.UnableToDestroySubscriptionFault;
import org.oasis_open.docs.wsn.bw_2.UnacceptableInitialTerminationTimeFault;
import org.oasis_open.docs.wsn.bw_2.UnrecognizedPolicyRequestFault;
import org.oasis_open.docs.wsn.bw_2.UnsupportedPolicyRequestFault;
import org.oasis_open.docs.wsrf.rpw_2.InvalidResourcePropertyQNameFault;
import org.oasis_open.docs.wsrf.rw_2.ResourceUnavailableFault;
import org.oasis_open.docs.wsrf.rw_2.ResourceUnknownFault;
import org.petalslink.abslayer.Factory;
import org.petalslink.abslayer.service.api.Binding;
import org.petalslink.abslayer.service.api.Description;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easybox.api.XmlContext;
import com.ebmwebsourcing.easybox.api.XmlContextFactory;
import com.ebmwebsourcing.easybox.api.XmlObjectReadException;
import com.ebmwebsourcing.easybox.api.XmlObjectReader;
import com.ebmwebsourcing.easybox.api.XmlObjectWriteException;
import com.ebmwebsourcing.easycommons.research.util.dom.DOMUtil;
import com.ebmwebsourcing.easycommons.research.util.easybox.SOAUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.ESBUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.EndpointAddress;
import com.ebmwebsourcing.easycommons.research.util.jaxb.SOAJAXBContext;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponent;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponentDeployBehaviour;
import com.ebmwebsourcing.easyesb.component.cd.api.CDProviderEndpoint;
import com.ebmwebsourcing.easyesb.component.cd.api.CDProviderEndpointBehaviour;
import com.ebmwebsourcing.easyesb.constant.EasyESBFramework;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.component.Component;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ClientEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.behaviour.EndpointBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.node.NodeBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.registry.RegistryEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.registry.RegistryEndpointBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.service.Service;
import com.ebmwebsourcing.easyesb.soa.api.service.ServiceBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;
import com.ebmwebsourcing.easyesb.soa.impl.component.AbstractComponentBehaviourImpl;
import com.ebmwebsourcing.easyesb.soa.impl.endpoint.ClientEndpointImpl;
import com.ebmwebsourcing.easyesb.soa.impl.service.BusinessServiceImpl;
import com.ebmwebsourcing.easyesb.soa10.api.element.EndpointInitialContext;
import com.ebmwebsourcing.easyesb.soa10.api.type.ClientEndpointType;
import com.ebmwebsourcing.easyesb.soa10.api.type.ComponentType;
import com.ebmwebsourcing.easyesb.soa10.api.type.ProviderEndpointType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import com.ebmwebsourcing.easywsdl11.api.element.Definitions;

import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeploy;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeployResponse;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeployementReport;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDescriptorType;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbExecutionEnvironmentInformation;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbExecutionEnvironmentInformationTypeType;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbUndeploy;
import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbUndeployResponse;
import easybox.org.oasis_open.docs.wsn.b_2.EJaxbNotify;
import easybox.org.oasis_open.docs.wsn.b_2.EJaxbSubscribe;
import easybox.org.oasis_open.docs.wsn.b_2.EJaxbSubscribeResponse;
import easybox.org.oasis_open.docs.wsn.b_2.EJaxbUnsubscribe;
import easybox.org.oasis_open.docs.wsn.b_2.EJaxbUnsubscribeResponse;
import easybox.org.oasis_open.docs.wsrf.rp_2.EJaxbGetResourcePropertyResponse;
import esstar.petalslink.com.data.management.admin._1.NodeInformation;
import esstar.petalslink.com.data.management.admin._1.Ping;
import esstar.petalslink.com.data.management.admin._1.PingResponse;
import esstar.petalslink.com.data.management.user._1.Bind;
import esstar.petalslink.com.data.management.user._1.BindResponse;
import esstar.petalslink.com.data.management.user._1.Proxify;
import esstar.petalslink.com.data.management.user._1.ProxifyResponse;
import esstar.petalslink.com.service.management._1_0.ManagementException;

/**
 * Class that provides the administration of the CD component, that is to say,
 * the deployment of CDs
 **/
public class CDComponentDeployBehaviourImpl extends AbstractComponentBehaviourImpl implements CDComponentDeployBehaviour {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(CDComponentDeployBehaviourImpl.class.getName());

	private static CoordinationDelegateUtility primitives = null;

	private static XmlContextFactory xmlContextFactory;
	private static XmlContext xmlContext;
	private static XmlObjectReader reader;

	private static Binding b;

	static {

		try {

			URL url = Thread.currentThread().getContextClassLoader().getResource(CDComponentDeployBehaviour.DESCRIPTION_URL);
			Description desc = (Description) Factory.getInstance().wrap(SOAUtil.getInstance().getReader(EasyESBFramework.getInstance()).get().readDocument(url, Definitions.class));
			b = desc.getBindings().iterator().next();

		} catch (XmlObjectReadException e) {
			e.printStackTrace();
			throw new RuntimeException();
		}
	}
	

	public CDComponentDeployBehaviourImpl(Component<? extends ComponentType> ep) {
		super(ep);
		this.setBinding(b);
		xmlContextFactory = new XmlContextFactory();
		xmlContext = xmlContextFactory.newContext();
		reader = xmlContext.createReader();
		primitives = new CoordinationDelegateUtility(this);

	}

	/**
	 * Main EASYESB component operation called when a request is sent to the CD
	 * administration service. In particular it receives deployment requests.
	 **/
	public void execute(Exchange exchange) throws TransportException {
		try {
			if (exchange.getMessageIn().getBody().getPayload() == null) {
				throw new TransportException("the message in cannot be null");
			}

			// convert dom to jdom
			DOMBuilder builder = new DOMBuilder();
			org.jdom.Document doc = builder.build(exchange.getMessageIn().getBody().getPayload());

			/*
			 * Deploy a CD
			 */
			if (doc.getRootElement() != null && doc.getRootElement().getName().equals("deploy")) {
				log.finest("Store CD");

				EJaxbDeploy deploy = SOAJAXBContext.getInstance().marshallAnyType(exchange.getMessageIn().getBody().getPayload(), EJaxbDeploy.class);

				/* call to the internal deploy operation */
				EJaxbDeployResponse res = this.deploy(deploy);

				Document docResp = SOAJAXBContext.getInstance().unmarshallAnyElement(res);
				MessageUtil.getInstance().createOutMessageStructure(exchange);
				exchange.getMessageOut().getBody().setPayload(docResp);

			} else
			/*
			 * Ping the service (checks its availability
			 */
			if (doc.getRootElement() != null && doc.getRootElement().getName().equals("ping")) {
				log.finest("Ping CD");

				Ping ping = SOAJAXBContext.getInstance().marshallAnyType(exchange.getMessageIn().getBody().getPayload(), Ping.class);

				PingResponse pong = this.ping(ping);

				Document docResp = SOAJAXBContext.getInstance().unmarshallAnyElement(pong);
				MessageUtil.getInstance().createOutMessageStructure(exchange);
				exchange.getMessageOut().getBody().setPayload(docResp);

			} else
			/*
			 * Undeploy a CD (NOT YET IMPLEMENTED)
			 */
			if (doc.getRootElement() != null &&

			doc.getRootElement().getName().equals("undeploy")) {

				log.warning("Undeploy CD Not Yet Implemented");

			}
		} catch (Exception e) {
			throw new TransportException(e);
		}

	}

	// DEPRECATED
	public List<ProviderEndpointType> store(URL bpelurl) throws ESBException {
		List<ProviderEndpointType> res = new ArrayList<ProviderEndpointType>();
		try {

			log.finest("store method");
			CDComponent bpelComponent = ((CDComponent) this.getEndpoint());

		} catch (Exception e) {
			log.severe("ERROR +e : " + e.getMessage());
			e.printStackTrace();
			throw new ESBException(e);
		}

		return res;
	}

	private URI createURIFromLocation(URL bpelurl, URI location) {
		URI res = null;
		String path = bpelurl.toString().substring(0, bpelurl.toString().lastIndexOf("/") + 1);

		res = URI.create(path + location.toString());
		return res;
	}

	private URI findLocationFromEndpoint(org.petalslink.abslayer.service.api.Endpoint ep, org.petalslink.abslayer.service.api.Description parentDescription) {
		URI res = null;
		org.petalslink.abslayer.service.api.Description desc = ep.getService().getDescription();
		for (org.petalslink.abslayer.service.api.Import impt : parentDescription.getImports()) {
			if (impt.getNamespace().equals(desc.getTargetNamespace())) {
				res = URI.create(impt.getLocation());
				break;
			}
			if (res == null) {
				res = findLocationFromEndpoint(ep, impt.getParentDescription());
			}
		}

		return res;
	}

	private boolean endpointNotExist(org.petalslink.abslayer.service.api.Endpoint ep, List<org.petalslink.abslayer.service.api.Endpoint> endpoints) {
		boolean res = false;
		for (org.petalslink.abslayer.service.api.Endpoint tmp : endpoints) {
			if (tmp.getName().equals(ep.getName()) && tmp.getService().getQName().getNamespaceURI().equals(ep.getService().getQName().getNamespaceURI()) && tmp.getService().getQName().getLocalPart().equals(ep.getService().getQName().getLocalPart())) {
				res = true;
				break;
			}
		}
		return res;
	}

	/**
	 * Deploy internal operation
	 **/
	@Override
	public EJaxbDeployResponse deploy(EJaxbDeploy dep) {

		/* Checks availability of input request */
		if (dep.getMainResource() == null || dep.getMainResource().getAttachment() == null || dep.getResources() == null || dep.getResources().isEmpty() || dep.getResources().size() < 2) {
			EJaxbDeployResponse faultRes = new EJaxbDeployResponse();
			EJaxbDeployementReport report = new EJaxbDeployementReport();
			EJaxbExecutionEnvironmentInformation info = new EJaxbExecutionEnvironmentInformation();
			info.setName("ERROR");
			report.setExecutionEnvironmentInformation(info);
			faultRes.setDeployementReport(report);
			return faultRes;
		}

		/* Find in additional resources the .properties file and the wsdl file */
		EJaxbDescriptorType cdPropertiesAttachment = null;
		EJaxbDescriptorType cdWsdl = null;

		if (dep.getResources().get(0).getFileURI().endsWith(".properties")) {
			cdPropertiesAttachment = dep.getResources().get(0);
			cdWsdl = dep.getResources().get(1);
		} else {
			cdPropertiesAttachment = dep.getResources().get(1);
			cdWsdl = dep.getResources().get(0);
		}

		try {

			/* Read the wsdl as a EasyWSDL definition */
			Definitions def = dealWithCDWsdl(cdWsdl);

			QName service = def.getServices()[0].inferQName();
			String endpoint = null;
			String endpointName = def.getServices()[0].getPorts()[0].getName();
			
			/* Extract and analyze CoordModel file */
			Coord coordModel = dealWithMainResource(dep.getMainResource());
			CoordinationDelegateData coordinationDelegateData = new CoordinationDelegateData();
			coordinationDelegateData.setCoordModel(coordModel);
			coordinationDelegateData.setInitialState(coordModel.getInitialState());
			
			coordinationDelegateData.setCoordinationChannelManager(new EasyESBCoordinationChannelManagerImpl((ClientEndpoint)this.endpoint));
			String coordinatedServiceEndpointName = endpointName;
			CoordinationDelegateFactory.addCoordinationDelegateData(coordinatedServiceEndpointName, coordinationDelegateData);
			
			/*
			 * Parse the CD properties file to know 1/ the name of the original
			 * WS enfpoint name 2/ the logical addresses of CDs to invoke
			 * (related to additional messages sending during coordination)
			 */
			dealWithCDProperties(cdPropertiesAttachment, def);

			/* Set deployment response */
			EJaxbDeployResponse res = new EJaxbDeployResponse();
			EJaxbDeployementReport report = new EJaxbDeployementReport();
			EJaxbExecutionEnvironmentInformation info = new EJaxbExecutionEnvironmentInformation();

			info.setName("cd deployment : " + service + "::" + endpointName);
			info.setType(EJaxbExecutionEnvironmentInformationTypeType.OTHERS);

			report.setExecutionEnvironmentInformation(info);
			res.setDeployementReport(report);
			return res;
		} catch (Exception e) {
			e.printStackTrace();
		}

		EJaxbDeployResponse res = new EJaxbDeployResponse();
		EJaxbDeployementReport report = new EJaxbDeployementReport();
		EJaxbExecutionEnvironmentInformation info = new EJaxbExecutionEnvironmentInformation();
		info.setName("cd deployment error ...");
		info.setType(EJaxbExecutionEnvironmentInformationTypeType.OTHERS);
		return res;

	}

	private Definitions dealWithCDWsdl(EJaxbDescriptorType cdWsdl) throws Exception {
		/* WSDL of CD loading */
		URI wsdlURI = URI.create(cdWsdl.getFileURI());
		System.out.println("wsdl uri : " + wsdlURI);
		System.out.println("wsdl string : " + cdWsdl.getFileURI());

		URL url;
		URI wsdlDescriptionAddress;

		if (wsdlURI.toString().contains("http:/")) {
			String tmp = wsdlURI.toString().split("http:/")[1];
			tmp = tmp.replace("%3F", "?");

			url = new URL("http://" + tmp);
			System.out.println("URL : " + url);

			wsdlDescriptionAddress = url.toURI();

		} else {
			System.out.println(wsdlURI);
			String[] split = wsdlURI.getPath().split("/");
			String wsdlFileName;
			if (split == null || split.length == 0) {
				wsdlFileName = wsdlURI.toString();
			} else {
				wsdlFileName = split[split.length - 1];
			}
			File wsdlFile = new File(System.getProperty("java.io.tmpdir"), wsdlFileName);
			wsdlFile.createNewFile();

			BufferedInputStream bis = new BufferedInputStream(cdWsdl.getAttachment().getInputStream());
			BufferedReader br = new BufferedReader(new InputStreamReader(bis));
			// DocumentBuilder dbu =
			// DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder();
			// Document wsdlDOM = dbu.parse(bis);
			FileOutputStream fos = new FileOutputStream(wsdlFile);
			String line;
			while ((line = br.readLine()) != null) {
				fos.write(line.getBytes());
			}
			url = wsdlFile.toURI().toURL();
			wsdlDescriptionAddress = wsdlFile.toURI();

		}
		Definitions def = readWSDL(url);

		URL cdGenericWSDL = Thread.currentThread().getContextClassLoader().getResource("wsdl/genericCDInterface.wsdl");

		QName service = def.getServices()[0].inferQName();
		String endpointName = def.getServices()[0].getPorts()[0].getName();
		QName interf = new QName(def.getTargetNamespace(), def.getPortTypes()[0].getName());
		JBITripletAddress cdMapKey = new JBITripletAddress(interf, service, endpointName);

		/* Create new endpoint attached to CD component (a new CD)) */

		RegistryEndpoint<?> registryEndpoint = (RegistryEndpoint<?>) this.getEndpoint().getNode().getRegistryEndpoint();
		if (registryEndpoint == null) {
			throw new ESBException("Impossible to find registry... Impossible to deploy CD. ");
		}

		if (registryEndpoint.findBehaviour(RegistryEndpointBehaviour.class).getEndpoint(ESBUtil.generateURI(new EndpointAddress(service, null))) == null) {

			Service newservice = this.createService(service, BusinessServiceImpl.class);

			EndpointInitialContext context = SOAUtil.getInstance().getXmlContext(EasyESBFramework.getInstance()).getXmlObjectFactory().create(EndpointInitialContext.class);
			context.setNumberOfThreads(5);

			CDProviderEndpoint providerEndpoint = ((ServiceBehaviour) newservice.findBehaviour(ServiceBehaviour.class)).createProviderEndpoint(def.getServices()[0].getPorts()[0].getName(), CDProviderEndpointImpl.class, CDProviderEndpointBehaviourImpl.class, context);

			providerEndpoint.addBehaviourClass(CDProviderEndpointBehaviourImpl.class);
			providerEndpoint.addBehaviourClass(CDGenericEndpointBehaviourImpl.class);
			providerEndpoint.refreshDescription();
			CDProviderEndpointBehaviour cdProviderEndpointBehaviour = providerEndpoint.findBehaviour(CDProviderEndpointBehaviour.class);

			Description desc = (Description) Factory.getInstance().wrap(def);

			// System.out.println("WSDL URI "+URI.create(oriWsdlUri));
			providerEndpoint.setWSDLDescriptionAddress(wsdlDescriptionAddress);

			Binding bi = desc.getBindings().iterator().next();
			cdProviderEndpointBehaviour.setBinding(bi);

			Document tmpdocbefore = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().newDocument();

			providerEndpoint.refreshDescription();

			try {
				SOAUtil.getInstance().getWriter(EasyESBFramework.getInstance()).get().writeDocument(providerEndpoint.getDescription().getModel(), tmpdocbefore);
			} catch (XmlObjectWriteException e) {
				e.printStackTrace();
			}

			System.out.println("WSDL CD  : " + providerEndpoint.getName() + "\n" + XMLPrettyPrinter.prettyPrint(tmpdocbefore));

			endpointName = providerEndpoint.getName();

			System.out.println("SERVICE ENDPOINT OF THE CD : " + service + " " + providerEndpoint.getWSDLDescriptionAddress());

		} else {
			log.warning("This CD already exists ..." + service + "::" + endpointName);
		}

		return def;

	}

	private void dealWithCDProperties(EJaxbDescriptorType cdPropertiesAttachment, Definitions def) throws IOException {
		/* CD configuration and adding in MAP <CDAddress, WSAddress> */

		Properties confCD = new Properties();

		confCD.load(cdPropertiesAttachment.getAttachment().getInputStream());// getAttachment().getInputStream());
		QName interf = new QName(def.getTargetNamespace(), def.getPortTypes()[0].getName());

		JBITripletAddress cdMapKey = new JBITripletAddress(interf, def.getServices()[0].inferQName(), def.getServices()[0].getPorts()[0].getName());
		JBITripletAddress cdMapValue = new JBITripletAddress(interf, def.getServices()[0].inferQName(), confCD.getProperty(CoordinationDelegateUtility.CONF_ENDPOINT_KEY));

		this.primitives.getCdToServiceMap().put(cdMapKey, cdMapValue);

	}

	private Coord dealWithMainResource(EJaxbDescriptorType mainResource) {
		try {
			String coordFileNameTemp = "coordtemp" + new Date().getTime();
			URL url;
			File coordFile = new File(System.getProperty("java.io.tmpdir"), coordFileNameTemp);
			coordFile.createNewFile();

			BufferedInputStream bis = new BufferedInputStream(mainResource.getAttachment().getInputStream());
			BufferedReader br = new BufferedReader(new InputStreamReader(bis));
			FileOutputStream fos = new FileOutputStream(coordFile);
			String line;
			while ((line = br.readLine()) != null) {
				fos.write(line.getBytes());
			}
			fos.flush();
			fos.close();
			url = coordFile.toURI().toURL();
			Coord coord = CoordmetamodelParser.parse(url.getPath());
			coordFile.delete();
			return coord;
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (MalformedURLException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return null;
	}

	@Override
	public PingResponse ping(Ping arg0) {
		PingResponse pong = new PingResponse();
		NodeInformation value = new NodeInformation();
		value.setName("CD Component on node " + this.getEndpoint().getNode().getName());
		value.setVersion("1.0-SNAPSHOT");
		pong.setNodeInformation(value);
		return pong;
	}

	public void createClientEndpoint(Holder<QName> clientEndpointName, String classClientEndpointName, String classClientEndpointBehaviourName) {
		Class<? extends ClientEndpointImpl> clazz;
		try {
			if (classClientEndpointName != null) {
				clazz = (Class<? extends ClientEndpointImpl<ClientEndpointType>>) Class.forName(classClientEndpointName);
			} else {
				clazz = (Class<ClientEndpointImpl>) ClientEndpointImpl.class;
			}

			// add behaviour
			Class<? extends EndpointBehaviour> clazzBehaviour = null;
			if (classClientEndpointBehaviourName != null && classClientEndpointBehaviourName.trim().length() > 0) {
				clazzBehaviour = (Class<? extends EndpointBehaviour>) Class.forName(classClientEndpointBehaviourName);
			}

			EndpointInitialContext context = SOAUtil.getInstance().getXmlContext(EasyESBFramework.getInstance()).getXmlObjectFactory().create(EndpointInitialContext.class);
			context.setNumberOfThreads(5);
			ClientEndpoint<ClientEndpointType> client = (ClientEndpoint<ClientEndpointType>) ((NodeBehaviour) this.getEndpoint().getNode().findBehaviour(NodeBehaviour.class)).createClientEndpoint(clientEndpointName.value, clazz, clazzBehaviour, context);
			client.addBehaviourClass(clazzBehaviour);

		} catch (Exception e) {
			e.printStackTrace();
		}

	}

	private void fillFile(File file, DataHandler dh) throws IOException {

		char[] buffer = new char[1024];
		Writer writer = new FileWriter(file);
		BufferedInputStream bis = new BufferedInputStream(dh.getInputStream());
		Reader reader = new BufferedReader(new InputStreamReader(bis, "UTF-8"));

		int n;
		while ((n = reader.read(buffer)) != -1) {

			writer.write(buffer, 0, n);

		}
		reader.close();
		bis.close();
		writer.flush();
		writer.close();
	}

	private static Definitions readWSDL(URL wsdlDOM) {
		Definitions def = null;

		try {
			def = reader.readDocument(wsdlDOM, Definitions.class);
		} catch (XmlObjectReadException e) {
			e.printStackTrace();
		}

		return def;
	}

	public CoordinationDelegateUtility getPrimitives() {
		return primitives;
	}

	public void setPrimitives(CoordinationDelegateUtility primitives) {
		this.primitives = primitives;
	}

	@Override
	public List<String> getSupportedResourcesExtensions() {
		List<String> res = new ArrayList<String>();
		return res;
	}

	@Override
	public BindResponse bind(Bind arg0) throws ManagementException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public String expose(QName arg0, String arg1) throws ManagementException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public ProxifyResponse proxify(Proxify arg0) throws ManagementException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void start() throws ESBException {
		// TODO Auto-generated method stub

	}

	@Override
	public void stop() throws ESBException {
		// TODO Auto-generated method stub

	}

	@Override
	public EJaxbUndeployResponse undeploy(EJaxbUndeploy arg0) throws ManagementException {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EJaxbGetResourcePropertyResponse getResourceProperty(QName arg0) throws ResourceUnavailableFault, InvalidResourcePropertyQNameFault, ResourceUnknownFault {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public void notify(EJaxbNotify arg0) {
		// TODO Auto-generated method stub

	}

	@Override
	public EJaxbSubscribeResponse subscribe(EJaxbSubscribe arg0) throws NotifyMessageNotSupportedFault, InvalidFilterFault, InvalidTopicExpressionFault, TopicExpressionDialectUnknownFault, UnsupportedPolicyRequestFault, SubscribeCreationFailedFault, UnacceptableInitialTerminationTimeFault, UnrecognizedPolicyRequestFault, TopicNotSupportedFault, InvalidProducerPropertiesExpressionFault, ResourceUnknownFault, InvalidMessageContentExpressionFault {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public EJaxbUnsubscribeResponse unsubscribe(EJaxbUnsubscribe arg0) throws UnableToDestroySubscriptionFault, ResourceUnknownFault {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public CoordinationDelegateAlgorithm getCoordinationDelegateAlgorithm(CoordinationDelegateID coordinationDelegateID) {
		//return CoordinationDelegateFactory.getCoordinationDelegateAlgorithm(coordinationDelegateID);
		return new DummyCoordinationDelegateAlgorithmImpl();
	}

}
