package com.ebmwebsourcing.easyesb.component.cd.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.AdditionalMessage;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;

import java.net.URL;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.choreos.genericcdinterface.AdditionalMessageType;
import org.choreos.genericcdinterface.Updatecurrentstate;
import org.petalslink.abslayer.Factory;
import org.petalslink.abslayer.service.api.Description;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.research.util.dom.DOMUtil;
import com.ebmwebsourcing.easycommons.research.util.easybox.SOAUtil;
import com.ebmwebsourcing.easycommons.research.util.jaxb.SOAJAXBContext;
import com.ebmwebsourcing.easyesb.component.cd.api.CDGenericEndpointBehaviour;
import com.ebmwebsourcing.easyesb.constant.EasyESBFramework;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.Endpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.behaviour.AbstractEndpointBehaviourImpl;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;
import com.ebmwebsourcing.easyesb.soa10.api.type.EndpointType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import com.ebmwebsourcing.easywsdl11.api.element.Definitions;

public class CDGenericEndpointBehaviourImpl extends AbstractEndpointBehaviourImpl implements CDGenericEndpointBehaviour {

	private static org.petalslink.abslayer.service.api.Binding cdBinding;
	private static Logger logger;

	private static final String UPDATE_CURRENT_STATE_OPERATION_NAME = "UpdateCurrentState";


	static {
		logger = java.util.logging.Logger.getLogger(CDGenericEndpointBehaviourImpl.class.getCanonicalName());
		URL cdGenericWSDL = Thread.currentThread().getContextClassLoader().getResource("wsdl/genericCDInterface.wsdl");
		cdBinding = parseCDGenericWSDL(cdGenericWSDL);
	}

	public CDGenericEndpointBehaviourImpl(Endpoint<? extends EndpointType> ep) {
		super(ep);
		this.setBinding(cdBinding);
	}

	public void execute(Exchange exchange) throws TransportException {
		logger.finest("[CD- Coordinatin Channel] Receiving Exchange " + exchange.getUuid() + " ... operation : " + exchange.getOperation() + "\n" + exchange.getMessageIn());
		Document p = exchange.getMessageIn().getBody().getPayload();
		logger.finest("[CD- Coordinatin Channel] payload is null ? " + p);

		assert (this.getBinding() != null);
		try {
			String operation = exchange.getOperation();
			/* Check if the request in the echange is accepted. */
			boolean found = false;
			for (org.petalslink.abslayer.service.api.BindingOperation bop : this.getBinding().getOperations()) {
				logger.finest("[CD- Coordinatin Channel] Looking for operation in binding " + bop.inferQName());
				if (bop.inferQName().equals(QName.valueOf(operation))) {
					found = true;
					break;
				}
			}
			if (found) {
				if (UPDATE_CURRENT_STATE_OPERATION_NAME.equals(operation)) {
					Updatecurrentstate updatecurrenteState = SOAJAXBContext.getInstance().marshallAnyType(exchange.getMessageIn().getBody().getPayload(), Updatecurrentstate.class);

					AdditionalMessage additionalMessage = createAdditionalMessage(updatecurrenteState.getAdditionalMessage());
					CoordinationDelegateID coordinationDelegateID = createCoordinationDelegateID(additionalMessage);
					CoordinationDelegateAlgorithm algorithm = CoordinationDelegateFactory.getCoordinationDelegateAlgorithm(coordinationDelegateID);
					algorithm.receiveUpdateCurrentState(additionalMessage);

				}

			} else {
				MessageUtil.getInstance().createErrorMessageStructure(exchange);
				Document errorDom = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().newDocument();
				Element error = errorDom.createElement("error");
				error.setTextContent("This exchange was not accepted by the CD Coordination Channel component.");
				errorDom.appendChild(error);
				exchange.getMessageError().getBody().setPayload(errorDom);
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new TransportException(e);
		}

	}

	private static org.petalslink.abslayer.service.api.Binding parseCDGenericWSDL(URL cdGenericWSDL) {
		org.petalslink.abslayer.service.api.Binding res = null;
		try {
			Description def = (Description) Factory.getInstance().wrap(SOAUtil.getInstance().getReader(EasyESBFramework.getInstance()).get().readDocument(cdGenericWSDL, Definitions.class));
			if (def.getBindings() != null && def.getBindings().size() > 0) {
				res = def.getBindings().iterator().next();
			}
		} catch (Exception e) {
			logger.severe("problem while creating the generic part of the CD behaviour (generic wsdl reading) ... " + e.getLocalizedMessage());
		}
		return res;
	}
	

	private AdditionalMessage createAdditionalMessage(AdditionalMessageType additionalMessageType) {
		AdditionalMessage additionalMessage = new AdditionalMessage();
		additionalMessage.setCoordinatedServiceEndpointName(additionalMessageType.getCoordinatedServiceEndpointName());
		additionalMessage.setChoreographyID(additionalMessageType.getChoreographyID());
		additionalMessage.setCoordinationDelegateFrom(additionalMessageType.getCoordinationDelegateFrom());
		additionalMessage.setCoordinationDelegateTo(additionalMessageType.getCoordinationDelegateTo());
		additionalMessage.setRequest(additionalMessageType.getRequest());
		additionalMessage.setState(additionalMessageType.getState());

		return additionalMessage;
	}
	
	private CoordinationDelegateID createCoordinationDelegateID(AdditionalMessage additionalMessage) {
		CoordinationDelegateID coordinationDelegateID = new CoordinationDelegateID();
		coordinationDelegateID.setCoordinatedServiceEndpointName(additionalMessage.getCoordinatedServiceEndpointName());
		coordinationDelegateID.setChoreographyID(additionalMessage.getChoreographyID());

		return coordinationDelegateID;
	}

}
