/*******************************************************************************
 * Copyright (c) 2011 EBM Websourcing.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the GNU Lesser Public License v2.1
 * which accompanies this distribution, and is available at
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.html
 * 
 * Contributors:
 *     EBM Websourcing - initial API and implementation
 ******************************************************************************/
package com.ebmwebsourcing.easyesb.component.cd.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationDelegateAlgorithm;
import it.univaq.di.choreos.coordinationprotocol.algorithm.DiscardException;
import it.univaq.di.choreos.coordinationprotocol.algorithm.ForwardAndDiscardMessagesCallback;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.ContextData;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.CoordinationDelegateID;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.AllowedOperation;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.CoordmetamodelFactory;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.coordmetamodel.Participant;

import java.net.URI;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.objectweb.fractal.api.Component;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

import com.ebmwebsourcing.easycommons.research.util.dom.DOMUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.ESBUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.EndpointAddress;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponent;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponentDeployBehaviour;
import com.ebmwebsourcing.easyesb.component.cd.api.CDProviderEndpointBehaviour;
import com.ebmwebsourcing.easyesb.exchange10.api.ExchangeException;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.exchange10.api.type.PatternType;
import com.ebmwebsourcing.easyesb.exchange10.api.type.RoleType;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ClientEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.Endpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ProviderEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.behaviour.AbstractEndpointBehaviourImpl;
import com.ebmwebsourcing.easyesb.soa.api.node.NodeBehaviour;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;
import com.ebmwebsourcing.easyesb.soa10.api.type.ComponentType;
import com.ebmwebsourcing.easyesb.soa10.api.type.EndpointType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;


/** 
 * Class implementing the behaviours of a CD service. For each deployed CD, a CDProviderEndpoint
 * with this behaviour is instanciated.
 *
 * On receiving a message dedicated to a WS for which a CD has been deployed, this class
 * applies the defined behaviour resulting from the combination of the global LTS and the one of the WS.
 * TODO: for now, systematically calls the unblock CD primitive, consisting in forwarding the message request and response.
 **/
public class CDProviderEndpointBehaviourImpl extends AbstractEndpointBehaviourImpl implements CDProviderEndpointBehaviour, ForwardAndDiscardMessagesCallback {


	
	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	private static Logger log = Logger.getLogger(CDProviderEndpointBehaviourImpl.class.getName());

	static private Integer currentIndReceiver = 0;
	

	public CDProviderEndpointBehaviourImpl(Endpoint<? extends EndpointType> arg0) {
		super(arg0);
		
	}


	@SuppressWarnings("unchecked")
	public void execute(Exchange exchange) throws TransportException {
		log.finest("[CD] Receiving Exchange "+exchange.getUuid()+" ... operation : "+exchange.getOperation()+"\n"+exchange.getMessageIn());
		Document p = exchange.getMessageIn().getBody().getPayload();
		log.finest("[CD] payload is null ? "+p);
		//log.finest("[CD] Payload : "+XMLPrettyPrinter.prettyPrint(p));
		assert(this.getBinding() != null);
		try {

			/* Check if the request in the echange is accepted. */
			boolean found = false;
			for(org.petalslink.abslayer.service.api.BindingOperation bop: this.getBinding().getOperations()) {
				log.finest("[CD] Looking for operation in binding "+bop.inferQName());
				if(bop.inferQName().equals(QName.valueOf(exchange.getOperation()))) {
					found = true;
					break;
				}
			}
			if (found) {
				/*
				if ("setInvocationAddress".equals(QName.valueOf(exchange.getOperation()).getLocalPart())) {
					
				}
				*/
				/* Check if a CD has been deployed for the particular message*/
				String destinationEndpoint = ESBUtil.analyzeURI(exchange.getDestinationReference()).getEndpointname();
				QName destinationService = exchange.getServiceName();
				QName destinationInterface = exchange.getInterfaceName();

				JBITripletAddress key = new JBITripletAddress(destinationInterface, destinationService, destinationEndpoint);

				ProviderEndpoint pe = ((ProviderEndpoint)this.endpoint);
				Component component = pe.getServiceProvider().getComponent();

				QName componentName = new QName(this.getEndpoint().getNode().getQName().getNamespaceURI(), CDComponent.NAME);
				EndpointAddress ea = new EndpointAddress(componentName, null);
				URI uri = ESBUtil.generateURI(ea);
				log.finest("[CD] URI of searched component : "+uri);

				System.out.println("References : ");
				for(com.ebmwebsourcing.easyesb.soa.api.component.Component<? extends ComponentType> c : ((NodeBehaviour)((ProviderEndpoint)this.endpoint).getNode().findBehaviour(NodeBehaviour.class)).getComponents()){
					log.finest("\t[CD] ref : "+c.getReference());
				}
				com.ebmwebsourcing.easyesb.soa.api.component.Component<? extends ComponentType> cd = ((NodeBehaviour)((ProviderEndpoint)this.endpoint).getNode().findBehaviour(NodeBehaviour.class)).getComponent(uri);//CDComponent.CD_QNAME);
				CDComponentDeployBehaviour cdComponentBehaviour = ((CDComponentDeployBehaviour) cd.findBehaviour(CDComponentDeployBehaviour.class));
				CoordinationDelegateUtility primitives = ((CDComponentDeployBehaviour)cd.findBehaviour(CDComponentDeployBehaviour.class)).getPrimitives();

				log.finest("[CD] key to find : "+key);
				log.finest("[CD] Map of CDs\n"+primitives.getCdToServiceMap());
				/* Get the endpoint of the real WS from the MAP */
				JBITripletAddress value = primitives.getCdToServiceMap().get(key);



				if(value != null){

					CoordinationDelegateID coordinationDelegateID = new CoordinationDelegateID(this.endpoint.getName(), uri.toString());
					CoordinationDelegateAlgorithm algorithm = cdComponentBehaviour.getCoordinationDelegateAlgorithm(coordinationDelegateID);
					CDContextData cdContextData = new CDContextData(exchange, value);
					ContextData contextData = new ContextData(cdContextData);
					Participant coordinationDelegateFrom = CoordmetamodelFactory.eINSTANCE.createParticipant();
					coordinationDelegateFrom.setName(this.endpoint.getName());
					AllowedOperation requestOperation = CoordmetamodelFactory.eINSTANCE.createAllowedOperation();
					requestOperation.setName(exchange.getOperation());
					algorithm.handleRules(requestOperation, coordinationDelegateFrom, this, contextData);
					//exchange.getMessageIn().getHeader().addProperty(MessageUtil.getInstance().createHeaderProperty(MessageUtil.EXCHANGE_ACCEPTED_BY_PROVIDER_PROPERTY, "true"));
				}else{
					MessageUtil.getInstance().createErrorMessageStructure(exchange);
					Document errorDom = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().newDocument();
					Element error = errorDom.createElement("error");
					error.setTextContent("This exchange was not accepted by the CD component.");
					errorDom.appendChild(error);
					exchange.getMessageError().getBody().setPayload(errorDom);
				}
			}
		} catch (Exception e) {
			e.printStackTrace();
			throw new TransportException(e);
		}
	}
	public void forwardMessage(ContextData contextData) {

		try {
			CDContextData cdContextData = (CDContextData) contextData.getData();
			Exchange exchange = cdContextData.getExchange();
			JBITripletAddress value = cdContextData.getValue();

			Exchange newExchange = ((ClientEndpoint) this.endpoint).createExchange();
			MessageUtil.getInstance().createInMessageStructure(newExchange);
			newExchange.setDestinationReference(ESBUtil.generateURI(new EndpointAddress(value.service, value.endpoint))); 
			newExchange.setServiceName(value.service);
			newExchange.setInterfaceName(value.interf);
			newExchange.setOperation(exchange.getOperation());
			newExchange.setPattern(exchange.getPattern());
			newExchange.setRole(RoleType.CONSUMER);
			newExchange.setSourceReference(exchange.getSourceReference());
			newExchange.setStatus(exchange.getStatus());
			newExchange.setUuid(newExchange.getUuid() + "CD");
			newExchange.getMessageIn().getBody().setPayload(exchange.getMessageIn().getBody().getPayload());
			forwardMessageInternal(exchange, newExchange, (ClientEndpoint) this.endpoint);
			// add specific header property to exchange
			// to precise that exchange is accepted by provider
			exchange.getMessageIn().getHeader().addProperty(MessageUtil.getInstance().createHeaderProperty(MessageUtil.EXCHANGE_ACCEPTED_BY_PROVIDER_PROPERTY, "true"));
		} catch (ExchangeException e) {
			log.severe("error "+e.getLocalizedMessage());
			throw new RuntimeException(e);
		} catch (ESBException e) {
			log.severe("error "+e.getLocalizedMessage());
			throw new RuntimeException(e);
		} catch (TransportException e) {
			log.severe("error "+e.getLocalizedMessage());
			throw new RuntimeException(e);
		}

	}

	public void discardMessage(ContextData contextData) throws DiscardException {
		// TODO Auto-generated method stub

	}

	
	private void forwardMessageInternal(Exchange exchange, Exchange newExchange, ClientEndpoint endpoint) throws ESBException, TransportException {
		if (newExchange.getPattern().equals(PatternType.IN_OUT)) {
			endpoint.sendSync(newExchange, 0);

			MessageUtil.getInstance().createOutMessageStructure(exchange);
			exchange.getMessageOut().getBody().setPayload(newExchange.getMessageOut().getBody().getPayload());
			endpoint.sendResponseToClient(exchange);

		} else {
			endpoint.send(newExchange);
		}

	}

	class CDContextData {
		private Exchange exchange;
		private JBITripletAddress value;

		public CDContextData(Exchange exchange, JBITripletAddress value) {
			super();
			this.exchange = exchange;
			this.value = value;
		}

		public Exchange getExchange() {
			return exchange;
		}

		public JBITripletAddress getValue() {
			return value;
		}

		public String toString() {
			return "CDContextData [exchange=" + exchange + ", value=" + value + "]";
		}
	}


}
