package com.ebmwebsourcing.easyesb.component.cd.impl;

import java.util.HashMap;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponentDeployBehaviour;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.exchange10.api.type.PatternType;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ClientEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;

/**
  * Class that implements Cooedination Algorithm primitives: block, unblock, ack, ...
  * For now, only unblock implemented.
 **/
public class CoordinationDelegatePrimitives {

	public final static String CONF_ENDPOINT_KEY = "endpoint";

	private Map<JBITripletAddress, JBITripletAddress> cdToServiceMap;
	
	private DocumentBuilder db = DocumentBuilders.takeDocumentBuilder();

	private CDComponentDeployBehaviour cdComponentBehaviour;
	
	public CoordinationDelegatePrimitives(CDComponentDeployBehaviour componentBehaviour) {
		this.cdComponentBehaviour = componentBehaviour;
		this.cdToServiceMap = new HashMap<JBITripletAddress, JBITripletAddress>();
		
	}

        /* unblock method: forward message form service requestor to the real service
	 *
	 * @exchange: exchange received by the CD. The out payload is to be forwarded to the original service requestor
	 * @newExchange: exchange between the CD and the original service requestor
	 * @endpoint: endpoint from which the newExchange is sent
	 */
	public void unblock(Exchange exchange, Exchange newExchange, ClientEndpoint endpoint) throws ESBException, TransportException {
		
	        /* InOut exchange, the response will be forwarded to the client */
		if(newExchange.getPattern().equals(PatternType.IN_OUT)){
			((ClientEndpoint)endpoint).sendSync(newExchange, 0);
			
			MessageUtil.getInstance().createOutMessageStructure(exchange);
			exchange.getMessageOut().getBody().setPayload(newExchange.getMessageOut().getBody().getPayload());
			endpoint.sendResponseToClient(exchange);
		
		}else
		/* InOnly exchange */
		{
			((ClientEndpoint)endpoint).send(newExchange);
		}

	}


	public  void block(Exchange exchange) throws ESBException {
			//....????
	}

	public void synchronize(Exchange exchange) throws ESBException{

	}

	public Map<JBITripletAddress, JBITripletAddress> getCdToServiceMap() {
		return cdToServiceMap;
	}

	

	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		
		DocumentBuilders.releaseDocumentBuilder(db);
	}
	
}
