package com.ebmwebsourcing.easyesb.component.cd.impl;

import it.univaq.di.choreos.coordinationprotocol.algorithm.CoordinationChannelManager;
import it.univaq.di.choreos.coordinationprotocol.algorithm.model.AdditionalMessage;

import java.net.URI;

import javax.xml.namespace.QName;

import org.choreos.genericcdinterface.AdditionalMessageType;
import org.choreos.genericcdinterface.Updatecurrentstate;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.research.util.esb.ESBUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.EndpointAddress;
import com.ebmwebsourcing.easycommons.research.util.jaxb.SOAJAXBContext;
import com.ebmwebsourcing.easyesb.exchange10.api.element.Exchange;
import com.ebmwebsourcing.easyesb.exchange10.api.type.PatternType;
import com.ebmwebsourcing.easyesb.exchange10.api.type.RoleType;
import com.ebmwebsourcing.easyesb.exchange10.api.type.StatusType;
import com.ebmwebsourcing.easyesb.soa.api.endpoint.ClientEndpoint;
import com.ebmwebsourcing.easyesb.soa.api.util.MessageUtil;

public class EasyESBCoordinationChannelManagerImpl implements CoordinationChannelManager {

	private ClientEndpoint endpoint;

	private static final String SERVICE_NAMESPACE = "http://choreos.org/genericCDInterface/";
	private static final String SERVICE_LOCALPART =  "GenericCDService";

	public EasyESBCoordinationChannelManagerImpl(ClientEndpoint endpoint) {
		this.endpoint = endpoint;
	}


	private void sendMessage(AdditionalMessage mess, String operationName){//URI value, String operation, PatternType pattern, URI sourceRef, StatusType status, Document payload) throws ExchangeException, TransportException{
		try {

			Exchange newExchange = ((ClientEndpoint)this.endpoint).createExchange();
			MessageUtil.getInstance().createInMessageStructure(newExchange);

			
			// TODO MATCH AdditionalMesage data structure with EasyESB one (missing stuff?)
			String choreId = mess.getChoreographyID();

			/* supposed to be the EasyESB logical address */
			String destinationCDEndpointName = mess.getCoordinationDelegateTo();
			EndpointAddress destinationAddress = new EndpointAddress(SERVICE_NAMESPACE,SERVICE_LOCALPART, destinationCDEndpointName);
			
			URI destinationCDEndpointURI = ESBUtil.generateURI(destinationAddress);
			newExchange.setDestinationReference(destinationCDEndpointURI);
			
			String sourceCDEndpointName = mess.getCoordinatedServiceEndpointName();
			EndpointAddress fromAddr = new EndpointAddress(SERVICE_NAMESPACE,SERVICE_LOCALPART, sourceCDEndpointName);
			URI sourceRef = ESBUtil.generateURI(fromAddr);
			
			PatternType pattern = PatternType.IN_ONLY;
			StatusType status = StatusType.ACTIVE;
			
			Updatecurrentstate u = new Updatecurrentstate();
			AdditionalMessageType messType = new AdditionalMessageType();
			messType.setChoreographyID(mess.getChoreographyID());
			messType.setCoordinatedServiceEndpointName(mess.getCoordinatedServiceEndpointName());
			messType.setCoordinationDelegateFrom(mess.getCoordinationDelegateFrom());
			messType.setCoordinationDelegateTo(mess.getCoordinationDelegateTo());
			messType.setRequest(mess.getRequest());
			messType.setState(mess.getState());
			u.setAdditionalMessage(messType);
			
			Document payload = SOAJAXBContext.getInstance().unmarshallAnyElement(u);

			newExchange.getMessageIn().getBody().setPayload(payload);
			newExchange.setServiceName(new QName(SERVICE_NAMESPACE, SERVICE_LOCALPART));
			newExchange.setOperation(operationName);
			newExchange.setPattern(pattern);
			newExchange.setRole(RoleType.CONSUMER);
			newExchange.setSourceReference(sourceRef);
			newExchange.setStatus(status);
			newExchange.setUuid(newExchange.getUuid()+""+operationName);
			if(newExchange.getPattern().equals(PatternType.IN_OUT)){
				((ClientEndpoint)endpoint).sendSync(newExchange, 0);

			}else {
				/* InOnly exchange */
				((ClientEndpoint)endpoint).send(newExchange);
			}

		} catch (Exception e) {

			e.printStackTrace();
		} 
	}

	@Override
	public void sendUpdateCurrentState(AdditionalMessage mess) {
		sendMessage(mess, "updatecurrentstate");
	}

}
