package com.ebmwebsourcing.easyesb.component.cd.test;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.util.Properties;
import java.util.logging.LogManager;

import javax.xml.namespace.QName;

import junit.framework.Assert;

import org.junit.Ignore;
import org.junit.Test;
import org.petalslink.abslayer.service.api.Service;

import starter.AirlineStarter;

import com.ebmwebsourcing.easycommons.research.util.esb.ESBUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.EndpointAddress;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponent;
import com.ebmwebsourcing.easyesb.component.cd.client.CDComponentClient;
import com.ebmwebsourcing.easyesb.component.cd.impl.ESBCDFactoryImpl;
import com.ebmwebsourcing.easyesb.external.protocol.soap.impl.server.SoapServer;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.component.Component;
import com.ebmwebsourcing.easyesb.soa.api.node.Node;
import com.ebmwebsourcing.easyesb.soa.api.node.NodeBehaviour;
import com.ebmwebsourcing.easyesb.soa.impl.config.ConfigurationImpl;
import com.ebmwebsourcing.easyesb.soa10.api.type.ComponentType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import com.ebmwebsourcing.esstar.management.UserManagementClient;

import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeployementReport;
import esstar.petalslink.com.service.management._1_0.ManagementException;

public class CDDeploymentTest {
	
	private static ESBCDFactoryImpl factory = new ESBCDFactoryImpl();
	private CDComponentClient cdClient;
	private UserManagementClient client;
	private String endpointService;

	private static String ADMIN_ADDRESS = "http://localhost:8285/services/adminExternalEndpoint";

	static {
		try {
			InputStream configFile = Thread.currentThread().getContextClassLoader().getResourceAsStream("easycommons-logging.properties");
			LogManager.getLogManager().readConfiguration(configFile);
		} catch (IOException ex) {
			System.out.println("WARNING: Could not open configuration file");
			System.out.println("WARNING: Logging not configured (console output only)");
		}

	}

	private Node createNode(QName name, boolean explorer, String host, int port, int soap_port) throws ESBException {

		ConfigurationImpl confImpl = new ConfigurationImpl(host, port);
		confImpl.getProperties().put(SoapServer.PORT_PROPERTY_NAME, String.valueOf(soap_port));
		Node node = factory.createNode(name, confImpl);

		return node;
	}

	/**
	 * This test runs the an example of deploying a CD 
	 **/
	//@Ignore
	@Test(timeout = 320000)
	public void testdeployCD() {

		Node node = null;
		try {
			/*
			 * Create EasyESB node with name node0, on localhost with
			 * transporter port 9010 (for internal exchanges) and soap port 8285
			 * (for services exposed as Web Services)
			 */
			node = createNode(new QName("http://com.ebmwebsourcing.easyesb", "node0"), false, "localhost", 9012, 8285);

			QName sn = CDComponent.SERVICE_QNAME;
			String en = null;
			EndpointAddress addr = new EndpointAddress(sn, en);

			/*
			 * Get CD Component to expose its administration (going to be
			 * useless now)
			 */
			Component<? extends ComponentType> cd = ((NodeBehaviour) node.findBehaviour(NodeBehaviour.class)).getComponent(ESBUtil.generateURI(addr));
			System.out.println("CD QName = " + cd);

			/* Create a client of the EasyESB administration */
			client = new UserManagementClient(ADMIN_ADDRESS);
			Service ser = cd.getDescription().getServices().iterator().next();
			System.out.println("Trying to Expose " + new QName(ser.getQName().getNamespaceURI(), CDComponent.NAME));

			/* Use the client to expose the CD administration service */
			String cdServiceExternalAddress = client.expose(new QName(ser.getQName().getNamespaceURI(), CDComponent.NAME), null);

			/* Create a client of the CD component administration */
			cdClient = new CDComponentClient(cdServiceExternalAddress);

			/* Load configurations for services (invocation addresses) */
			Properties realAddressProp = new Properties();
			realAddressProp.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("partners-noESB-noCD.properties"));

			Properties invocationAddressProp = new Properties();
			invocationAddressProp.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("partners-withESB-noCD.properties"));
			AirlineStarter airlineStarter = new AirlineStarter(realAddressProp.getProperty("airline"), invocationAddressProp, null);
			
			deployCD("deployment-test" + File.separator + "test.xmi", "deployment-test" + File.separator + "test.wsdl", "deployment-test" + File.separator + "test.properties", "http://localhost:9111/airline", new QName("http://services.choreos.org/", "AirlineService"), "CDAirline");


		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		} finally {
			try {
				if (node != null) {
					node.stop();
				}
			} catch (TransportException e) {
				Assert.fail();
			}
		}

	}

	

	public String deployCD(String coordModel, String cdWsdl, String prop, String realServiceAddress, QName service, String endpoint) throws URISyntaxException, ManagementException {

		File wsdlFile = new File(Thread.currentThread().getContextClassLoader().getResource(cdWsdl).toURI());
		File coordModelFile = new File(Thread.currentThread().getContextClassLoader().getResource(coordModel).toURI());
		File propertiesFile = new File(Thread.currentThread().getContextClassLoader().getResource(prop).toURI());

		/* 1) deploy CD of WS1 */
		EJaxbDeployementReport report = cdClient.deploy(coordModelFile, wsdlFile, propertiesFile);
		System.out.println("CD Deployement report : " + report.getExecutionEnvironmentInformation().getName());

		/* 2) bind WS1 */
		client.bind(realServiceAddress, realServiceAddress + "?wsdl");

		/*
		 * 3) expose the CD as a new service (resulting external WS address is
		 * sent as a response of the function)
		 */
		endpointService = client.expose(service, endpoint);

		/* External address of the CD */
		return endpointService;
	}
}
