package com.ebmwebsourcing.easyesb.component.cd.test;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.ServerSocket;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.logging.LogManager;

import javax.xml.namespace.QName;
import javax.xml.parsers.ParserConfigurationException;

import junit.framework.Assert;

import org.jdom.JDOMException;
import org.junit.Ignore;
import org.junit.Test;
import org.petalslink.abslayer.service.api.Service;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import starter.AirlineGroundStaffMIDStarter;
import starter.AirlineStarter;
import starter.AirtportStarter;
import starter.GroundTransportationCompanyStarter;
import starter.HotelStarter;
import starter.StandAndGateManagementStarter;
import starter.TravelAgencyStarter;
import starter.WeatherForecastServiceStarter;

import com.ebmwebsourcing.easycommons.research.util.dom.DOMUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.ESBUtil;
import com.ebmwebsourcing.easycommons.research.util.esb.EndpointAddress;
import com.ebmwebsourcing.easycommons.soap.handler.SOAPException;
import com.ebmwebsourcing.easycommons.soap.handler.SOAPSender;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;
import com.ebmwebsourcing.easyesb.component.cd.api.CDComponent;
import com.ebmwebsourcing.easyesb.component.cd.client.CDComponentClient;
import com.ebmwebsourcing.easyesb.component.cd.impl.ESBCDFactoryImpl;
import com.ebmwebsourcing.easyesb.external.protocol.soap.impl.server.SoapServer;
import com.ebmwebsourcing.easyesb.soa.api.ESBException;
import com.ebmwebsourcing.easyesb.soa.api.component.Component;
import com.ebmwebsourcing.easyesb.soa.api.node.Node;
import com.ebmwebsourcing.easyesb.soa.api.node.NodeBehaviour;
import com.ebmwebsourcing.easyesb.soa.impl.config.ConfigurationImpl;
import com.ebmwebsourcing.easyesb.soa10.api.type.ComponentType;
import com.ebmwebsourcing.easyesb.transporter.api.transport.TransportException;
import com.ebmwebsourcing.esstar.management.UserManagementClient;

import easybox.esstar.petalslink.com.management.model.datatype._1.EJaxbDeployementReport;
import esstar.petalslink.com.service.management._1_0.ManagementException;

public class CDRuntimeComponentTest {

	private static ESBCDFactoryImpl factory = new ESBCDFactoryImpl();// "data-collector-factory",
																		// "wsdm-monitoring-factory",
																		// "sla-manager-factory");
	CDComponentClient cdClient;
	UserManagementClient client;
	String endpointService;

	private SOAPSender ss = new SOAPSender();

	private static String ADMIN_ADDRESS = "http://localhost:8285/services/adminExternalEndpoint";

	static {
		try {
			InputStream configFile = Thread.currentThread().getContextClassLoader().getResourceAsStream("easycommons-logging.properties");
			LogManager.getLogManager().readConfiguration(configFile);
		} catch (IOException ex) {
			System.out.println("WARNING: Could not open configuration file");
			System.out.println("WARNING: Logging not configured (console output only)");
		}

	}

	private Node createNode(QName name, boolean explorer, String host, int port, int soap_port) throws ESBException {

		ConfigurationImpl confImpl = new ConfigurationImpl(host, port);
		confImpl.getProperties().put(SoapServer.PORT_PROPERTY_NAME, String.valueOf(soap_port));
		Node node = factory.createNode(name, confImpl);

		return node;
	}

	/**
	 * This test runs the flight use case (mid-review demo) choreography. The
	 * choreography is ran in EasyESB with CDs for each involved service.
	 * 
	 * The validation is made by counting the number of calls to a particular
	 * service "airlinegroundstaffmid" (number of booked amentities)
	 **/
	@Ignore
	@Test(timeout = 320000)
	public void testUnblockMessage() {

		Node node = null;
		try {
			/*
			 * Create EasyESB node with name node0, on localhost with
			 * transporter port 9010 (for internal exchanges) and soap port 8285
			 * (for services exposed as Web Services)
			 */
			node = createNode(new QName("http://com.ebmwebsourcing.easyesb", "node0"), false, "localhost", 9012, 8285);

			QName sn = CDComponent.SERVICE_QNAME;
			String en = null;
			EndpointAddress addr = new EndpointAddress(sn, en);

			/*
			 * Get CD Component to expose its administration (going to be
			 * useless now)
			 */
			Component<? extends ComponentType> cd = ((NodeBehaviour) node.findBehaviour(NodeBehaviour.class)).getComponent(ESBUtil.generateURI(addr));
			System.out.println("CD QName = " + cd);

			/* Create a client of the EasyESB administration */
			client = new UserManagementClient(ADMIN_ADDRESS);
			Service ser = cd.getDescription().getServices().iterator().next();
			System.out.println("Trying to Expose " + new QName(ser.getQName().getNamespaceURI(), CDComponent.NAME));

			/* Use the client to expose the CD administration service */
			String cdServiceExternalAddress = client.expose(new QName(ser.getQName().getNamespaceURI(), CDComponent.NAME), null);

			/* Create a client of the CD component administration */
			cdClient = new CDComponentClient(cdServiceExternalAddress);

			/* Load configurations for services (invocation addresses) */
			Properties realAddressProp = new Properties();
			realAddressProp.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("partners-noESB-noCD.properties"));

			Properties invocationAddressProp = new Properties();
			invocationAddressProp.load(Thread.currentThread().getContextClassLoader().getResourceAsStream("partners-withESB-noCD.properties"));

			// int port = 9111;
			// while(!isPortAvailable(port)){
			// System.out.println("Wait for the "+port+" port to be available...");
			// Thread.sleep(1000);
			//
			// }

			/* Start services and set the choreography context */
			AirlineStarter airlineStarter = new AirlineStarter(realAddressProp.getProperty("airline"), invocationAddressProp, null);

			AirlineGroundStaffMIDStarter airlineGroundStaffMidStarter = new AirlineGroundStaffMIDStarter(realAddressProp.getProperty("airlinegroundstaffmid"), invocationAddressProp);

			WeatherForecastServiceStarter weatherforcasstarter = new WeatherForecastServiceStarter(realAddressProp.getProperty("weatherforecastservice"), invocationAddressProp, 2, 4);

			TravelAgencyStarter travelAgencyStarter = new TravelAgencyStarter(realAddressProp.getProperty("travelagency"), invocationAddressProp);

			AirtportStarter airportStarter = new AirtportStarter(realAddressProp.getProperty("airport"), invocationAddressProp);

			StandAndGateManagementStarter standandgatemanagementStarter = new StandAndGateManagementStarter(realAddressProp.getProperty("standandgatemanagement"), invocationAddressProp);

			HotelStarter hotelStarter = new HotelStarter(realAddressProp.getProperty("hotel"), invocationAddressProp, null);

			GroundTransportationCompanyStarter groundstationcompanyStarter = new GroundTransportationCompanyStarter(realAddressProp.getProperty("groundtransportationcompany"), invocationAddressProp);

			/*
			 * Deploy CD for each service using utility function
			 * prepareServiceCDTuple (see comments in it)
			 */
			String serviceAddressAirlinecd = this.prepareServiceCDTuple("cdwsdl/cdairline.wsdl", "lts/airline.lts", "http://localhost:9111/airline", new QName("http://services.choreos.org/", "AirlineService"), "CDAirlinePort", "coord.properties/airline.properties");

			String serviceAddressAirlineGroundStaffcd = this.prepareServiceCDTuple("cdwsdl/cdairlinegroundstaffmid.wsdl", "lts/airlinegroundstaffmid.lts", "http://localhost:9111/airlinegroundstaffmid", new QName("http://services.choreos.org/", "AirlineGroundStaffMIDService"), "CDAirlineGroundStaffMIDPort", "coord.properties/airlinegroundstaffmid.properties");

			String serviceTravelAgencycd = this.prepareServiceCDTuple("cdwsdl/cdtravelagency.wsdl", "lts/travelagency.lts", "http://localhost:9111/travelagency", new QName("http://services.choreos.org/", "TravelAgencyService"), "CDTravelAgencyPort", "coord.properties/travelagency.properties");

			String serviceWeatherForecastcd = this.prepareServiceCDTuple("cdwsdl/cdweatherforecastservice.wsdl", "lts/weatherforecastservice.lts", "http://localhost:9111/weatherforecastservice", new QName("http://services.choreos.org/", "WeatherForecastServiceService"), "CDWeatherForecastServicePort", "coord.properties/weatherforecastservice.properties");

			String serviceAirportcd = this.prepareServiceCDTuple("cdwsdl/cdairport.wsdl", "lts/airport.lts", "http://localhost:9111/airport", new QName("http://services.choreos.org/", "AirportService"), "CDAirportPort", "coord.properties/airport.properties");

			String serviceHotelcd = this.prepareServiceCDTuple("cdwsdl/cdhotel.wsdl", "lts/hotel.lts", "http://localhost:9111/hotel", new QName("http://services.choreos.org/", "HotelService"), "CDHotelPort", "coord.properties/hotel.properties");

			String serviceGroundtransportationcompanycd = this.prepareServiceCDTuple("cdwsdl/cdgroundtransportationcompany.wsdl", "lts/groundtransportationcompany.lts", "http://localhost:9111/groundtransportationcompany", new QName("http://services.choreos.org/", "GroundTransportationCompanyService"), "CDGroundTransportationCompanyPort", "coord.properties/groundtransportationcompany.properties");

			String serviceStandandgatemanagementcd = this.prepareServiceCDTuple("cdwsdl/cdstandandgatemanagement.wsdl", "lts/standandgatemanagement.lts", "http://localhost:9111/standandgatemanagement", new QName("http://services.choreos.org/", "StandandgatemanagementService"), "CDStandandgatemanagementPort", "coord.properties/standandgatemanagement.properties");

			/* Populate the invocation list with cd addresses */
			Properties invocationAddresses = new Properties();
			invocationAddresses.put("airline", serviceAddressAirlinecd);
			invocationAddresses.put("airlinegroundstaffmid", serviceAddressAirlineGroundStaffcd);
			invocationAddresses.put("travelagency", serviceTravelAgencycd);
			invocationAddresses.put("airport", serviceAirportcd);
			invocationAddresses.put("hotel", serviceHotelcd);
			invocationAddresses.put("groundtransportationcompany", serviceGroundtransportationcompanycd);
			invocationAddresses.put("standandgatemanagement", serviceStandandgatemanagementcd);
			invocationAddresses.put("weatherforecastservice", serviceWeatherForecastcd);

			HashMap<String, String> invocationAddressesMap = new HashMap<String, String>((Map) invocationAddresses);

			airlineStarter.getServiceAirline().setChoreographyContext(invocationAddressesMap);
			airlineGroundStaffMidStarter.getservice().setChoreographyContext(invocationAddressesMap);
			airportStarter.getservice().setChoreographyContext(invocationAddressesMap);
			groundstationcompanyStarter.getservice().setChoreographyContext(invocationAddressesMap);
			hotelStarter.getservice().setChoreographyContext(invocationAddressesMap);
			standandgatemanagementStarter.getservice().setChoreographyContext(invocationAddressesMap);
			travelAgencyStarter.getservice().setChoreographyContext(invocationAddressesMap);
			weatherforcasstarter.getservice().setChoreographyContext(invocationAddressesMap);

			System.out.println(invocationAddressesMap);

			/* Start the choreography */
			airlineStarter.getServiceAirline().warnReroute("12", null);

			/*
			 * Wait until the right number of calls to getNbAmenityReceipt is
			 * made Timeout must be set on unit test not to be blocked in case
			 * of test failure.
			 */
			while (airlineGroundStaffMidStarter.getservice().getNbAmeniyReceiptCalls("get") < 3) {
				System.out.println("get Number of anemities: " + airlineGroundStaffMidStarter.getservice().getNbAmeniyReceiptCalls("get"));
				Thread.sleep(1000);
			}

			airlineStarter.getEndpointAirline().stop();
			airlineGroundStaffMidStarter.getEndpoint().stop();
			airportStarter.getEndpoint().stop();
			groundstationcompanyStarter.getEndpoint().stop();
			hotelStarter.stop();
			standandgatemanagementStarter.getEndpoint().stop();
			travelAgencyStarter.getEndpoint().stop();
			weatherforcasstarter.getEndpoint().stop();
			node.stop();
			// String airlinegroundstaffmidURL =
			// airlineStarter.getServiceAirline().invocationAddresses.get("airlinegroundstaffmid");
			// System.out.println("Calling on URL: "+airlinegroundstaffmidURL);
			// while(getNbAmeniyReceiptCalls(airlinegroundstaffmidURL)<3){
			// System.out.println(getNbAmeniyReceiptCalls(airlinegroundstaffmidURL));
			// System.out.println(getNbBookingReceiptCalls(airlinegroundstaffmidURL));
			// Thread.sleep(3000);
			// }
			// Thread.sleep(1000000);

		} catch (Exception e) {
			e.printStackTrace();
			Assert.fail();
		} finally {
			try {
				if (node != null) {
					node.stop();
				}
			} catch (TransportException e) {
				Assert.fail();
			}
		}

	}

	private int getNbBookingReceiptCalls(String url) throws SAXException, IOException, ParserConfigurationException, SOAPException, JDOMException {
		InputStream reqURL = Thread.currentThread().getContextClassLoader().getResourceAsStream("requests/getNbBookingReceiptCalls.xml");
		Document req = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().parse(reqURL);
		Document res = ss.sendSoapRequest(SOAPSender.createSOAPMessageRequest(req), url, null);

		System.out.println(XMLPrettyPrinter.prettyPrint(res));

		return -1;
	}

	private int getNbAmeniyReceiptCalls(String url) throws SAXException, IOException, ParserConfigurationException, SOAPException, JDOMException {

		InputStream reqURL = Thread.currentThread().getContextClassLoader().getResourceAsStream("requests/getNbAmeniyReceiptCalls.xml");
		Document req = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().parse(reqURL);
		Document res = ss.sendSoapRequest(SOAPSender.createSOAPMessageRequest(req), url, null);

		System.out.println(XMLPrettyPrinter.prettyPrint(res));

		return -1;

	}

	private boolean isPortAvailable(int port) {
		boolean isAvailable = false;
		ServerSocket socket = null;
		try {
			socket = new ServerSocket(port);
			isAvailable = true;
		} catch (IOException e) {
			isAvailable = false;
		} finally {
			if (socket != null)
				try {
					socket.close();
				} catch (IOException e) { /* e.printStackTrace(); */
				}

		}
		return isAvailable;
	}

	/*
	 * Utility function that 1) deploy a CD for WS1 2) bind the WS1 service to
	 * the EasyESB (import WS as an EasyESB service) 3) expose the CD service as
	 * a new WS (proxifying the WS1 one)
	 */
	public String prepareServiceCDTuple(String cdWsdl, String cdLts, String realServiceAddress, QName service, String endpoint, String prop) throws URISyntaxException, ManagementException {

		System.out.println("cdWsdl " + cdWsdl + " cdLts" + cdLts + " realServiceAddress" + realServiceAddress + " service " + service + " endpoint" + endpoint + " prop : " + prop);

		File wsdlFile = new File(Thread.currentThread().getContextClassLoader().getResource(cdWsdl).toURI());
		File confFile = new File(Thread.currentThread().getContextClassLoader().getResource(cdLts).toURI());
		File propertiesFile = new File(Thread.currentThread().getContextClassLoader().getResource(prop).toURI());

		/* 1) deploy CD of WS1 */
		EJaxbDeployementReport report = cdClient.deploy(confFile, wsdlFile, propertiesFile);
		System.out.println("CD Deployement report : " + report.getExecutionEnvironmentInformation().getName());

		/* 2) bind WS1 */
		client.bind(realServiceAddress, realServiceAddress + "?wsdl");

		/*
		 * 3) expose the CD as a new service (resulting external WS address is
		 * sent as a response of the function)
		 */
		endpointService = client.expose(service, endpoint);

		/* External address of the CD */
		return endpointService;
	}
}