package org.petalslink.easiestdemo.client;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

import javax.swing.ImageIcon;
import javax.swing.JEditorPane;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.xml.parsers.ParserConfigurationException;

import org.ow2.petals.notifier.NotificationConsumer_NotifierEndpoint_Server;
import org.petalslink.easiestdemo.client.model.api.ws.MockOperation;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.research.util.dom.DOMUtil;
import com.ebmwebsourcing.easycommons.research.util.easybox.SOAUtil;
import com.ebmwebsourcing.easycommons.soap.handler.SOAPException;
import com.ebmwebsourcing.easycommons.soap.handler.SOAPSender;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;
import com.ebmwebsourcing.easyesb.exchange10.api.type.PatternType;


public class ThreadMsgSender extends Thread {

    private SOAPSender sender = new SOAPSender(); 

    private MockOperation operation;
    private JLabel image = null;
    private JEditorPane request = null;
    private JEditorPane response = null;
    private String sendTo = null;
    private WSOUIClient client = null;

    public ThreadMsgSender(MockOperation operation, JLabel image, JEditorPane request, JEditorPane response, String sendTo, WSOUIClient client) {
        this.operation = operation;
        this.image = image;
        this.request = request;
        this.response = response;
        this.sendTo = sendTo;
        this.client = client;
    }

    @Override
    public void run() {
        this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_TEL_IMAGE));
        this.image.paint(this.image.getGraphics());

        MockOperation bop = (MockOperation) this.operation;
        try {
        	InputStream is = new ByteArrayInputStream(this.request.getText().getBytes());
            Document request = DOMUtil.getInstance().getDocumentBuilderFactory().newDocumentBuilder().parse(is);
            request = analyseRequest(request);
            Document response = this.sender.sendSoapRequest(request, sendTo, bop.getSoapAction());
            is.close();
            if(PatternType.IN_OUT.equals(bop.getPattern())){
                this.response.setText(XMLPrettyPrinter.prettyPrint(response));
            }

            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_HAPPY_IMAGE));
            this.image.paint(this.image.getGraphics());


            this.client.getRegistry().refreshConnectionBetweenFramework();
        } catch (SOAPException e) {
            this.response.setText(e.getMessage());
            e.printStackTrace();
            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_SAD_IMAGE));
            this.image.paint(this.image.getGraphics());
        } catch (SAXException e) {
            this.response.setText(e.getMessage());
            e.printStackTrace();
            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_SAD_IMAGE));
            this.image.paint(this.image.getGraphics());
        } catch (IOException e) {
            this.response.setText(e.getMessage());
            e.printStackTrace();
            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_SAD_IMAGE));
            this.image.paint(this.image.getGraphics());
        } catch (ParserConfigurationException e) {
            this.response.setText(e.getMessage());
            e.printStackTrace();
            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_SAD_IMAGE));
            this.image.paint(this.image.getGraphics());
        } catch (WSOUIClientException e) {
            this.response.setText(e.getMessage());
            e.printStackTrace();
            this.image.setIcon(new ImageIcon(WSOUIClient.CLIENT_SAD_IMAGE));
            this.image.paint(this.image.getGraphics());
        }
    }	

    private Document analyseRequest(Document request) throws WSOUIClientException {
        Element body = (Element) request.getDocumentElement().getElementsByTagNameNS("http://schemas.xmlsoap.org/soap/envelope/", "Body").item(0);
        if(body != null) {
            Element operation = (Element) DOMUtil.getInstance().getFirstElement(body);
            if(operation != null && operation.getLocalName().equals("Subscribe")) {
                Element consumerReference = (Element) DOMUtil.getInstance().getFirstElement(operation);
                Element addressElmt = (Element) DOMUtil.getInstance().getFirstElement(consumerReference);
                String addressString = XMLPrettyPrinter.prettyPrint(addressElmt).replace("<?xml version=\"1.0\" encoding=\"UTF-8\"?>", "").trim();

                addressString = addressString.substring(addressString.indexOf(">")+1, addressString.lastIndexOf("<"));
                String address = addressString.trim();
                if(!findIfAddressExist(address)) {
                    // create notification server address
                    int ok = JOptionPane.showConfirmDialog(null, "Noticifation server to received notification does not exist.\nDo you want create it?", "Question",
                            JOptionPane.YES_NO_OPTION);
                    if(ok == JOptionPane.YES_OPTION) {
                        this.client.addNewNotificationServers(address);
                    }
                }
            }
        }
        return request;
    }

    private boolean findIfAddressExist(String address) {
        boolean res = false;
        for(NotificationConsumer_NotifierEndpoint_Server server: this.client.getNotificationServers()) {
            if(server.getAddress().equals(address)) {
                res = true;
                break;
            }
        }
        return res;
    }




}
