package org.ow2.petals.cd.engine;

import java.util.HashMap;
import java.util.Map;

import javax.jbi.messaging.MessagingException;
import javax.xml.parsers.DocumentBuilder;

import org.ow2.petals.cd.engine.listener.JBIListener;
import org.ow2.petals.cd.engine.message.JBIContextImpl;
import org.ow2.petals.cd.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.cd.engine.util.JBITripletAddress;
import org.ow2.petals.cd.engine.util.Sender;
import org.ow2.petals.cd.engine.util.Sender.SendStyle;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.Message.MEPConstants;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.w3c.dom.Document;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.XMLPrettyPrinter;

public class CoordinationDelegatePrimitives {

	private AbstractComponent cdComponent = null;

	private JBIListener jbiListener = null;

	private Map<JBITripletAddress, JBITripletAddress> cdToServiceMap;
	
	private DocumentBuilder db = DocumentBuilders.takeDocumentBuilder();

	public CoordinationDelegatePrimitives(AbstractComponent component) {
		this.cdComponent = component;
		this.cdToServiceMap = new HashMap<JBITripletAddress, JBITripletAddress>();
		
	}

	public void unblock(Exchange exchange) throws MessagingException {

		final Sender sender = new Sender(this.cdComponent.getContext(), this.cdComponent.getChannel(), this.cdComponent.getLogger());
		final JBIExternalMessageImpl message = new JBIExternalMessageImpl();

		JBITripletAddress cdaddress = new JBITripletAddress(exchange.getInterfaceName(), exchange.getService(), exchange.getEndpointName());

		JBITripletAddress serviceAddress = this.cdToServiceMap.get(cdaddress);
		
		if(serviceAddress != null){

			message.setEndpoint(serviceAddress.endpoint);
			message.setService(exchange.getService());
			message.setOperationName(exchange.getOperation().getLocalPart());
			boolean isInOnly = exchange.isInOnlyPattern();
			MEPConstants mep;
			if(isInOnly){
				mep = MEPConstants.IN_ONLY_PATTERN;
			}else{
				mep = MEPConstants.IN_OUT_PATTERN;
			}
			org.w3c.dom.Document doc = exchange.getInMessageContentAsDocument();
			//org.w3c.dom.Document childDoc = Util.getFirstChildAsDocument(doc);
			System.out.println("MESSAGE FORWARDED ...\n"+XMLPrettyPrinter.prettyPrint(doc));
			message.setContent(doc.getDocumentElement());

			this.cdComponent.getLogger().info("Going to forward message received to "+ serviceAddress.endpoint);
			//TODO FIND ENDPOINT IN LTS
			//SEND MESSAGE

			
			try {
				
				JBIContextImpl context = new JBIContextImpl(exchange, sender, this.jbiListener);
				
				JBIExternalMessageImpl outMessage = sender.send(message, new Long(0), serviceAddress, message.getOperationName(), SendStyle.SYNCHRONOUS_SEND, context);
				
				if(mep.equals(MEPConstants.IN_OUT_PATTERN)&&(outMessage != null) && (outMessage.getContent() != null)){
					Document outDom = db.newDocument();
					outDom.adoptNode(outMessage.getContent());
					outDom.appendChild(outMessage.getContent());
					exchange.setOutMessageContent(outDom);
					
				}else if(mep.equals(MEPConstants.IN_ONLY_PATTERN)){
					exchange.setDoneStatus();
				}else{
					throw new PEtALSCDKException("In-Out call sent fault ...");
				}
				
			} catch (PEtALSCDKException e) {
				
				throw new MessagingException(e.getLocalizedMessage());
			}
		}else{
			throw new MessagingException("Endpoint not found for the received message ...");
		}

	}


	public  void block(Exchange exchange) throws MessagingException {
			//....????
	}

	public void synchronize(Exchange exchange) throws MessagingException{

	}

	public Map<JBITripletAddress, JBITripletAddress> getCdToServiceMap() {
		return cdToServiceMap;
	}

	public JBIListener getJbiListener() {
		return jbiListener;
	}

	public void setJbiListener(JBIListener jbiListener) {
		this.jbiListener = jbiListener;
	}



	@Override
	protected void finalize() throws Throwable {
		super.finalize();
		
		DocumentBuilders.releaseDocumentBuilder(db);
	}
	
}
