/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.cd.engine.su;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.List;
import java.util.Properties;

import javax.xml.namespace.QName;

import org.ow2.petals.cd.engine.CDEngine;
import org.ow2.petals.cd.engine.util.JBITripletAddress;
import org.ow2.petals.cd.engine.util.Util;
import org.ow2.petals.component.framework.AbstractComponent;
import org.ow2.petals.component.framework.api.configuration.ConfigurationExtensions;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.jbidescriptor.generated.Jbi;
import org.ow2.petals.component.framework.jbidescriptor.generated.Provides;
import org.ow2.petals.component.framework.su.AbstractServiceUnitManager;


/**
 * This class implements the orchestra su manager
 *
 * @author Nicolas SALATGE - eBMWebSourcing
 *
 */
public final class CDServiceUnitManager extends AbstractServiceUnitManager{// implements ServiceUnitListener {

	/**
	 * LTS properties
	 */
	private static final String LTS_PROPERTY = "lts";

	private static final String LTS_ENDPOINT = "endpoint";

	private static final String LTS_SERVICE = "service";

	private static final String LTS_INTERFACE = "interface";

	/**
	 * Default constructor.
	 * @param component
	 */
	public CDServiceUnitManager(AbstractComponent component) {
		super(component);
	}


	@Override
	protected void doDeploy(final String serviceUnitName, final String suRootPath,
			final Jbi jbiDescriptor) throws PEtALSCDKException {
		try {

			// 
			final List<Provides> providesList = jbiDescriptor.getServices()
					.getProvides();

			for (final Provides provides : providesList) {
				final ConfigurationExtensions props = new ConfigurationExtensions(
						provides.getAny());

				// get parameters from extension
				final String ltsFileName = props
						.get(CDServiceUnitManager.LTS_PROPERTY);

				JBITripletAddress cdAddress = new JBITripletAddress(provides.getInterfaceName(), provides.getServiceName(), provides.getEndpointName());

				JBITripletAddress serviceAddress = extractJBIAddressFromLTS(ltsFileName, suRootPath, provides);

				((CDEngine)this.component).getPrimitives().getCdToServiceMap().put(cdAddress, serviceAddress);
			}

			this.logger
			.info("Coordination Delegate Service Unit deployment has been performed.");
		} catch (final Exception e) {
			this.logger.severe("CD deployment failed ! see the error message: "+e.getMessage());
			throw new PEtALSCDKException(e);
		}
	}

	private JBITripletAddress extractJBIAddressFromLTS(String ltsFileName, String suRootPath, Provides provides) {

		JBITripletAddress res = new JBITripletAddress();

		try{

			Properties prop = new Properties();
			URI ltsURI = Util.getResourceAsURL(suRootPath, ltsFileName).toURI();
			File ltsFile = new File(ltsURI);

			FileInputStream input = new FileInputStream(ltsFile);
			prop.load(input);

			res.endpoint = prop.getProperty(LTS_ENDPOINT);
			if(prop.getProperty(LTS_SERVICE) != null){
				res.service = QName.valueOf(prop.getProperty(LTS_SERVICE));
			}else{
				res.service = provides.getServiceName();
			}
			
			if(prop.getProperty(LTS_INTERFACE) != null){
				res.interf = QName.valueOf(prop.getProperty(LTS_INTERFACE));
			}else{
				res.interf = provides.getInterfaceName();
			}
		}catch (URISyntaxException e) {
			this.logger.severe("Failed to parse URI of lts file ...");
		} catch (MalformedURLException e) {
			this.logger.severe("Failed to parse URL of lts file ...");
		} catch (FileNotFoundException e) {
			this.logger.severe("LTS File not found ...");
		} catch (IOException e) {
			this.logger.severe("Problem while loading LTS filel as properties ...");
		}catch (Exception e) {
			e.printStackTrace();
		}
		return res;
	}


}

