/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM WebSourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.cd.engine.util;

import java.io.IOException;
import java.net.URI;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessagingException;
import javax.jbi.servicedesc.ServiceEndpoint;
import javax.xml.namespace.QName;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.transform.dom.DOMSource;

import org.ow2.petals.cd.engine.listener.JBIListener;
import org.ow2.petals.cd.engine.message.JBIContextImpl;
import org.ow2.petals.cd.engine.message.JBIExternalMessageImpl;
import org.ow2.petals.component.framework.api.Message;
import org.ow2.petals.component.framework.api.exception.PEtALSCDKException;
import org.ow2.petals.component.framework.api.message.Exchange;
import org.ow2.petals.component.framework.util.ExchangeUtil;
import org.ow2.petals.component.framework.util.NormalizedMessageUtil;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;
import com.ebmwebsourcing.easycommons.xml.SourceHelper;

/**
 * @author nicolas salatge - ebmm websourcing
 * @author alouis
 *
 */
public class Sender {

	private final Logger logger;


	private final ComponentContext componentContext;

	/**
	 * Send style
	 *
	 */
	public static enum SendStyle {
		ASYNCHRONOUS_SEND, SYNCHRONOUS_SEND, SYNCHRONOUS_SEND_WITH_TIMEOUT
	};

	/**
	 * Default constructor
	 *
	 */
	public Sender(final ComponentContext context,
			final DeliveryChannel channel, final Logger logger) {

		this.logger = logger;

		this.componentContext = context;
	}

	/**
	 * FIXME some orchestra Error and Fault exception are thrown FIXME use some
	 * CDK utils to send Exchanges Call a jbi service
	 *
	 * @param partner
	 *            the partner to invoke. May be null
	 * @param msgIn
	 *            the exchange
	 * @param syncTime
	 *            timeout for syncSend
	 * @return the exchange response
	 * @throws CoreException
	 * @throws MessagingException
	 *             problem while accessing MsgExch information or while sending
	 *             the MsgExc
	 * @throws OrchestraEngineException no endpoint found to send the message
	 */
	public final JBIExternalMessageImpl send(final JBIExternalMessageImpl internalMsg,
			final Long syncTime, JBITripletAddress providerEndpointInfo, String operationName, final SendStyle sendStyle, JBIContextImpl context) throws PEtALSCDKException {
		JBIExternalMessageImpl messageReturned = null;
		try {
			if(providerEndpointInfo == null) {
				throw new PEtALSCDKException("The provider endpoint information cannot be null");
			}

			// retrieve the jbi context
			final JBIContextImpl jbiContext = context;
			JBIListener listener = jbiContext.getListener();

			System.out.println("create JBI exchange");

			Exchange msgIn = this.createMsg(providerEndpointInfo.interf, operationName, sendStyle, internalMsg,
					jbiContext.getInitialExchange(), listener);


			String endpointName = null;
			QName service = null;

			msgIn.setInterfaceName(providerEndpointInfo.interf);

			service = providerEndpointInfo.service;
			System.out.println("invoked service name: "+service);

			endpointName = providerEndpointInfo.endpoint;
			System.out.println("invoked endpoint name: " + endpointName);




			if (service == null && endpointName == null){
				System.out.println("Service name and endpoint not set, working only on interface.");
			}else if (service != null) {

				System.out.println("Set the service to invoke: " + service);
				msgIn.setService(service);
				if (endpointName != null && ! "autogenerate".equals(endpointName)) {
					final ServiceEndpoint serviceEndpoint = this.resolveEndPoint(service,
							endpointName);

					System.out.println("Set the endpoint to invoke: "
							+ serviceEndpoint.getEndpointName());
					msgIn.setEndpoint(serviceEndpoint);
				}
				System.out.println("invoke: " + endpointName);
			} 

			System.out.println("Satus: " + msgIn.getStatus());
			System.out.println("Consumer Role? " + msgIn.isConsumerRole());

			if ((sendStyle == SendStyle.ASYNCHRONOUS_SEND)
					&& (msgIn.isInOnlyPattern())) {
				listener.send(msgIn);
			} else if (sendStyle == SendStyle.SYNCHRONOUS_SEND) {

				listener.sendSync(msgIn);
				messageReturned = internalMsg;
				messageReturned.setContent(msgIn.getOutMessageContentAsDocument().getDocumentElement());

			}
		}catch (Exception e) {
			e.printStackTrace();
			this.logger.severe("Error while sending exchange ... "+e.getLocalizedMessage());
		}
		return messageReturned;
	}







	/**
	 * Create a JBI message exchange
	 * @param itf
	 * @param operation
	 * @param sendStyle
	 * @param content
	 * @param initialExchange
	 * @param listener
	 * @return
	 * @throws BPELException
	 * @throws PEtALSCDKException
	 */
	private Exchange createMsg(final QName itf, final String operation, final SendStyle sendStyle,
			JBIExternalMessageImpl message, final Exchange initialExchange, JBIListener listener)
					throws PEtALSCDKException {
		Exchange exchange = null;
		try {
			Message.MEPConstants mep = Message.MEPConstants.IN_ONLY_PATTERN;

			if (sendStyle == SendStyle.SYNCHRONOUS_SEND) {
				mep = Message.MEPConstants.IN_OUT_PATTERN;
			}


			exchange = this.createExchange(mep, listener);
			exchange.setInterfaceName(itf);

			exchange.setOperation(new QName(itf.getNamespaceURI(), operation));
			exchange.setInMessageContent(message.getContent().getOwnerDocument());

			if (initialExchange.getInMessage().getSecuritySubject() != null) {
				exchange.getInMessage().setSecuritySubject(
						initialExchange.getInMessage().getSecuritySubject());
			}

			// Copy attachments
			NormalizedMessageUtil.copyAttachments(initialExchange.getInMessage(), exchange.getInMessage());

			ExchangeUtil.copyExchangeProperties(initialExchange, exchange);


		} catch (final MessagingException e) {
			throw new PEtALSCDKException(e);
		}
		return exchange;
	}


	/**
	 * FIXME use some CDK util methods to create the exchange Create an empty
	 * Exchange
	 *
	 * @param mep
	 *            exchange pattern
	 * @return
	 * @throws MessagingException
	 *             JBI factory can not create the MsgExchange
	 */
	public Exchange createExchange(final Message.MEPConstants mep, JBIListener listener) throws MessagingException {
		if (mep == null) {
			throw new IllegalArgumentException(
					"'mep' parameter must be non null");
		}
		Exchange exchange = null;
		try {
			if(mep.equals(Message.MEPConstants.IN_ONLY_PATTERN)) {
				exchange = listener.createExchange(Message.MEPConstants.IN_ONLY_PATTERN);
			} else  if(mep.equals(Message.MEPConstants.IN_OUT_PATTERN)) {
				exchange = listener.createExchange(Message.MEPConstants.IN_OUT_PATTERN);
			} else  if(mep.equals(Message.MEPConstants.ROBUST_IN_ONLY_PATTERN)) {
				exchange = listener.createExchange(Message.MEPConstants.ROBUST_IN_ONLY_PATTERN);
			} else  if(mep.equals(Message.MEPConstants.IN_OPTIONAL_OUT_PATTERN)) {
				exchange = listener.createExchange(Message.MEPConstants.IN_OPTIONAL_OUT_PATTERN);
			}
		} catch (PEtALSCDKException e) {
			throw new MessagingException(e);
		}
		return exchange;
	}

	public final ServiceEndpoint resolveEndPoint(final QName service,
			final String endpointName) throws PEtALSCDKException {

		this.logger.log(Level.FINEST, "find the endPoint in the JBI environment");

		this.logger.log(Level.FINEST, "service name: " + service);
		this.logger.log(Level.FINEST, "endpoint name: " + endpointName);

		// Search by endpoint                                                                                                                                                                     
		final ServiceEndpoint endpoint = this.componentContext.getEndpoint(service, endpointName);

		if (endpoint == null) {
			throw new PEtALSCDKException("endpoint (" + service + "-"
					+ endpointName + ") not found in the JBI environment");
		}

		this.logger.log(Level.FINEST, "endpoint service found: " + endpoint);
		return endpoint;
	}


}

