/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2006 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.cd.engine.util;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @author Nicolas Salatge - eBM WebSourcing
 */
public class Util {

    /**
     * We retrieve the URL associated to the resource
     *
     * @param resourceName
     *            the resource name
     * @return the URL
     * @throws MalformedURLException
     *             impossible to generate url
     */
    public static URL getResourceAsURL(final String rootPath, final String resourceName)
    throws MalformedURLException {
        try {
            return new URL(resourceName);
        } catch(final MalformedURLException e) {
            return new File(rootPath + "/" + resourceName).toURI().toURL();
        }
    }

    private static Element getFirstElement(Document doc){
        Element res = null;
        NodeList list = doc.getFirstChild().getChildNodes();
        for(int i=0 ; i < list.getLength() ; i++){
            if(list.item(i) instanceof Element){
                res = (Element)list.item(i);
            }
        }
        return res;
    }

    public static Document getFirstChildAsDocument(Document doc){
        Document res = null;

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);

        Element el = Util.getFirstElement(doc);
        if(el != null){
            Node n = el.cloneNode(true);

            try {
                res = factory.newDocumentBuilder().newDocument();

                res.adoptNode(n);
                res.importNode(n, true);
                res.appendChild((Element)n);

            } catch (ParserConfigurationException e) {
                throw new RuntimeException(e);
            }
        }

        return res;
    }
}
