package com.ebmwebsourcing.petals.services.generation.cd;

import java.io.File;
import java.net.URL;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.ebmwebsourcing.easybox.api.XmlContext;
import com.ebmwebsourcing.easybox.api.XmlContextFactory;
import com.ebmwebsourcing.easybox.api.XmlObjectReadException;
import com.ebmwebsourcing.easybox.api.XmlObjectReader;
import com.ebmwebsourcing.easywsdl11.api.element.Definitions;
import com.ebmwebsourcing.petals.services.generation.AbstractJbiXml;
import com.ebmwebsourcing.petals.services.generation.JbiBeanDelegate;
import com.ebmwebsourcing.petals.services.generation.JbiUtils;
import com.ebmwebsourcing.petals.services.generation.cdk5.components.CDProvides10;

public final class CDSAGeneratorUtil {

	// Read WSDL 1.1 definitions
	private static Definitions readDefinitions (URL wsdlUrl) throws XmlObjectReadException {
		// create factory: can be static
		XmlContextFactory xmlContextFactory = new XmlContextFactory();

		// create context: can be static
		XmlContext xmlContext = xmlContextFactory.newContext();

		// create generic reader: cannot be static!!! not thread safe!!!
		XmlObjectReader reader = xmlContext.createReader();

		// read WSDL 1.1 definitions from url
		Definitions definitions = reader.readDocument(wsdlUrl, Definitions.class);
		return definitions;
	}

	
	static public File generateSA(File installFolder, URL wsdlURL, URL ltsURL) throws Exception{
		
		File res = null;
		
	try {
			
			installFolder.mkdir();
			
			String componentName = "petals-se-cd";
			
			CDProvides10 cdBean = new CDProvides10();
			AbstractJbiXml cdDel = new AbstractJbiXml();
			cdDel.setComponentName(componentName);

			/* LTS */
			String ltsFileName = ltsURL.getFile().substring(ltsURL.getFile().lastIndexOf("/")+1);		

			/* WSDL */
			String wsdlFileName = wsdlURL.getFile().substring(ltsURL.getFile().lastIndexOf("/")+1);
			
			Definitions suWsdlDefinitions = readDefinitions(wsdlURL);

			/* Common info to provide for SU */
			//TODO Assume using first interface, first service, first endpoint, and that wsdl targetnamespace is used for interface and service.
			cdBean.setEndpointName(suWsdlDefinitions.getServices()[0].getPorts()[0].getName());
			cdBean.setInterfaceName(suWsdlDefinitions.getPortTypes()[0].getName());
			cdBean.setInterfaceNsUri(suWsdlDefinitions.getTargetNamespace());
			cdBean.setServiceName(suWsdlDefinitions.getServices()[0].getName());
			cdBean.setServiceNsUri(suWsdlDefinitions.getTargetNamespace());

			cdBean.getFilesToImport().put(ltsFileName, new File(ltsURL.toURI()));
			cdBean.getFilesToImport().put(wsdlFileName, new File(wsdlURL.toURI()));

			cdBean.setLtsFileName(ltsFileName);
			cdBean.setWsdl(wsdlFileName);

			List<JbiBeanDelegate> delList = new ArrayList<JbiBeanDelegate>();
			delList.add(cdBean);

			/* SU Creation */
			String jbixmlOfSU = cdDel.createJbiDescriptor(delList).toString();

			String suName = JbiUtils.createSuName(cdBean.getSuType(), cdBean.getServiceName(), false);

			
			File suRes = new File(installFolder, suName+".zip");

			suRes = JbiUtils.createJbiArchive(suRes.getAbsolutePath(), jbixmlOfSU, cdBean.getFilesToImport());

			/* Sa Creation */
			String saName = JbiUtils.createSaName(suName);
			Map<String, String> suNameToComponentName = new HashMap<String, String>();
			suNameToComponentName.put(suName, componentName);

			Map<String, File> suNameTosuFile = new HashMap<String, File>();
			suNameTosuFile.put(suName+".zip", suRes);

			String jbixmlOfSA = JbiUtils.generateJbiXmlForSA(saName, suNameToComponentName);

			File saRes = new File(installFolder, saName+".zip");
			saRes = JbiUtils.createJbiArchive(saRes.getAbsolutePath(), jbixmlOfSA, suNameTosuFile);			
			
			res = saRes;
		} catch (Exception e) {
			e.printStackTrace();
			throw e;
		}
		
		
		
		
		
		
		return res;
		
	}
	
}
